Package: yaml
Type: Package
Title: Methods to Convert R Data to YAML and Back
Date: 2024-07-22
Version: 2.3.10
Suggests: RUnit
Author: Shawn P Garbett [aut], Jeremy Stephens [aut, cre], Kirill Simonov [aut], Yihui Xie [ctb],
  Zhuoer Dong [ctb], Hadley Wickham [ctb], Jeffrey Horner [ctb], reikoch [ctb],
  Will Beasley [ctb], Brendan O'Connor [ctb], Gregory R. Warnes [ctb],
  Michael Quinn [ctb], Zhian N. Kamvar [ctb], Charlie Gao [ctb]
Maintainer: Shawn Garbett <shawn.garbett@vumc.org>
License: BSD_3_clause + file LICENSE
Description: Implements the 'libyaml' 'YAML' 1.1 parser and emitter
  (<https://pyyaml.org/wiki/LibYAML>) for R.
URL: https://github.com/vubiostat/r-yaml/
BugReports: https://github.com/vubiostat/r-yaml/issues
NeedsCompilation: yes
Packaged: 2024-07-22 15:44:18 UTC; garbetsp
Repository: CRAN
Date/Publication: 2024-07-26 15:10:02 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 00:56:04 UTC; unix

Type: Package
Package: yaml
Title: Methods to Convert R Data to YAML and Back
Version: 2.3.12
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = "cre",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Shawn", "Garbett", , "shawn.garbett@vumc.org", role = "ctb",
           comment = c(ORCID = "0000-0003-4079-5621")),
    person("Jeremy", "Stephens", role = c("aut", "ctb")),
    person("Kirill", "Simonov", role = "aut"),
    person("Yihui", "Xie", role = "ctb",
           comment = c(ORCID = "0000-0003-0645-5666")),
    person("Zhuoer", "Dong", role = "ctb"),
    person("Jeffrey", "Horner", role = "ctb"),
    person("reikoch", role = "ctb"),
    person("Will", "Beasley", role = "ctb",
           comment = c(ORCID = "0000-0002-5613-5006")),
    person("Brendan", "O'Connor", role = "ctb"),
    person("Michael", "Quinn", role = "ctb"),
    person("Charlie", "Gao", role = "ctb"),
    person(c("Gregory", "R."), "Warnes", role = "ctb"),
    person(c("Zhian", "N."), "Kamvar", role = "ctb")
  )
Description: Implements the 'libyaml' 'YAML' 1.1 parser and emitter
    (<https://pyyaml.org/wiki/LibYAML>) for R.
License: BSD_3_clause + file LICENSE
URL: https://yaml.r-lib.org, https://github.com/r-lib/yaml/
BugReports: https://github.com/r-lib/yaml/issues
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2025-12-08 16:53:07 UTC; hadleywickham
Author: Hadley Wickham [cre] (ORCID: <https://orcid.org/0000-0003-4757-117X>),
  Shawn Garbett [ctb] (ORCID: <https://orcid.org/0000-0003-4079-5621>),
  Jeremy Stephens [aut, ctb],
  Kirill Simonov [aut],
  Yihui Xie [ctb] (ORCID: <https://orcid.org/0000-0003-0645-5666>),
  Zhuoer Dong [ctb],
  Jeffrey Horner [ctb],
  reikoch [ctb],
  Will Beasley [ctb] (ORCID: <https://orcid.org/0000-0002-5613-5006>),
  Brendan O'Connor [ctb],
  Michael Quinn [ctb],
  Charlie Gao [ctb],
  Gregory R. Warnes [ctb],
  Zhian N. Kamvar [ctb]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2025-12-10 07:00:01 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:39:54 UTC; unix
Archs: yaml.so.dSYM

# Generated by roxygen2: do not edit by hand

export(as.yaml)
export(read_yaml)
export(verbatim_logical)
export(write_yaml)
export(yaml.load)
export(yaml.load_file)
useDynLib(yaml, .registration = TRUE, .fixes = "C_")

<!-- README.md is generated from README.Rmd. Please edit that file -->

# yaml

<!-- badges: start -->

[![R-CMD-check](https://github.com/r-lib/yaml/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/yaml/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/r-lib/yaml/graph/badge.svg)](https://app.codecov.io/gh/r-lib/yaml)
<!-- badges: end -->

yaml provides R bindings to [libyaml](https://pyyaml.org/wiki/LibYAML),
a fast [YAML](https://yaml.org/) parser and emitter.

## Installation

Install from CRAN:

``` r
install.packages("yaml")
```

Or install the development version from GitHub:

``` r
# install.packages("pak")
pak::pak("r-lib/r-yaml")
```

## Usage

``` r
library(yaml)
```

Parse YAML with `yaml.load()` or `read_yaml()`:

``` r
yaml.load(
  "
- 1
- 2
- 3
"
)
#> [1] 1 2 3

yaml.load(
  "
a: 1
b: 2
"
)
#> $a
#> [1] 1
#> 
#> $b
#> [1] 2
```

Convert R objects to YAML with `as.yaml()` or `write_yaml()`:

``` r
cat(as.yaml(list(a = 1:3, b = 4:6)))
#> a:
#> - 1
#> - 2
#> - 3
#> b:
#> - 4
#> - 5
#> - 6
```

See `vignette("yaml")` for more details on handlers, formatting options,
and advanced usage.

NAME
    YAML - YAML Ain't Markup Language™

STATUS
    YAML is undergoing changes that will make it the future gateway to
    almost all (new and old) YAML functionality, while at the same time
    working almost exactly the same for existing code.

    See
    <https://github.com/ingydotnet/yaml-old-pm/blob/master/doc/yaml-old-tran
    sition.md#yaml-to-yamlold-transition> for details.

        use YAML;

        my $yaml = "foo: 42\n";
        my $hash = Load $yaml;
        $hash->{bar} = 44;
        $yaml = Dump $hash;

DESCRIPTION
    YAML is a human friendly data serialization language with
    implementations in many programming languages.

    YAML.pm is Perl 5's first YAML module. Written in 2001, that code has
    been moved to YAML::Old an YAML is now the API frontend to various YAML
    backends.

    NOTE: This is a current WIP and more doc will follow as things develop.

AUTHORS
    Ingy döt Net <ingy@cpan.org>
    Tina Müller <cpan2@tinita.de>

COPYRIGHT AND LICENSE
    Copyright 2001-2017. Ingy döt Net.

    This program is free software; you can redistribute it and/or modify it
    under the same terms as Perl itself.

    See <http://www.perl.com/perl/misc/Artistic.html>

NAME
    YAML - YAML Ain't Markup Language (tm)

NOTE
    This module has been released to CPAN as YAML::Old, and soon YAML.pm
    will be changed to just be a frontend interface module for all the
    various Perl YAML implementation modules, including YAML::Old.

    If you want robust and fast YAML processing using the normal Dump/Load
    API, please consider switching to YAML::XS. It is by far the best Perl
    module for YAML at this time. It requires that you have a C compiler,
    since it is written in C.

    If you really need to use this version of YAML.pm it will always be
    available as YAML::Old.

    If you don't care which YAML module use, as long as it's the best one
    installed on your system, use YAML::Any.

    The rest of this documentation is left unchanged, until YAML.pm is
    switched over to the new UI-only version.

SYNOPSIS
        use YAML;
        
    # Load a YAML stream of 3 YAML documents into Perl data structures.
        my ($hashref, $arrayref, $string) = Load(<<'...');
        ---
        name: ingy
        age: old
        weight: heavy
        # I should comment that I also like pink, but don't tell anybody.
        favorite colors:
            - red
            - green
            - blue
        ---
        - Clark Evans
        - Oren Ben-Kiki
        - Ingy döt Net
        --- >
        You probably think YAML stands for "Yet Another Markup Language". It
        ain't! YAML is really a data serialization language. But if you want
        to think of it as a markup, that's OK with me. A lot of people try
        to use XML as a serialization format.
        
    "YAML" is catchy and fun to say. Try it. "YAML, YAML, YAML!!!"
        ...
        
    # Dump the Perl data structures back into YAML.
        print Dump($string, $arrayref, $hashref);
        
    # YAML::Dump is used the same way you'd use Data::Dumper::Dumper
        use Data::Dumper;
        print Dumper($string, $arrayref, $hashref);

DESCRIPTION
    The YAML.pm module implements a YAML Loader and Dumper based on the YAML
    1.0 specification. <http://www.yaml.org/spec/>

    YAML is a generic data serialization language that is optimized for
    human readability. It can be used to express the data structures of most
    modern programming languages. (Including Perl!!!)

    For information on the YAML syntax, please refer to the YAML
    specification.

WHY YAML IS COOL
    YAML is readable for people.
        It makes clear sense out of complex data structures. You should find
        that YAML is an exceptional data dumping tool. Structure is shown
        through indentation, YAML supports recursive data, and hash keys are
        sorted by default. In addition, YAML supports several styles of
        scalar formatting for different types of data.

    YAML is editable.
        YAML was designed from the ground up to be an excellent syntax for
        configuration files. Almost all programs need configuration files,
        so why invent a new syntax for each one? And why subject users to
        the complexities of XML or native Perl code?

    YAML is multilingual.
        Yes, YAML supports Unicode. But I'm actually referring to
        programming languages. YAML was designed to meet the serialization
        needs of Perl, Python, Ruby, Tcl, PHP, Javascript and Java. It was
        also designed to be interoperable between those languages. That
        means YAML serializations produced by Perl can be processed by
        Python.

    YAML is taint safe.
        Using modules like Data::Dumper for serialization is fine as long as
        you can be sure that nobody can tamper with your data files or
        transmissions. That's because you need to use Perl's "eval()"
        built-in to deserialize the data. Somebody could add a snippet of
        Perl to erase your files.

        YAML's parser does not need to eval anything.

    YAML is full featured.
        YAML can accurately serialize all of the common Perl data structures
        and deserialize them again without losing data relationships.
        Although it is not 100% perfect (no serializer is or can be
        perfect), it fares as well as the popular current modules:
        Data::Dumper, Storable, XML::Dumper and Data::Denter.

        YAML.pm also has the ability to handle code (subroutine) references
        and typeglobs. (Still experimental) These features are not found in
        Perl's other serialization modules.

    YAML is extensible.
        The YAML language has been designed to be flexible enough to solve
        it's own problems. The markup itself has 3 basic construct which
        resemble Perl's hash, array and scalar. By default, these map to
        their Perl equivalents. But each YAML node also supports a tagging
        mechanism (type system) which can cause that node to be interpreted
        in a completely different manner. That's how YAML can support object
        serialization and oddball structures like Perl's typeglob.

YAML IMPLEMENTATIONS IN PERL
    This module, YAML.pm, is really just the interface module for YAML
    modules written in Perl. The basic interface for YAML consists of two
    functions: "Dump" and "Load". The real work is done by the modules
    YAML::Dumper and YAML::Loader.

    Different YAML module distributions can be created by subclassing
    YAML.pm and YAML::Loader and YAML::Dumper. For example, YAML-Simple
    consists of YAML::Simple YAML::Dumper::Simple and YAML::Loader::Simple.

    Why would there be more than one implementation of YAML? Well, despite
    YAML's offering of being a simple data format, YAML is actually very
    deep and complex. Implementing the entirety of the YAML specification is
    a daunting task.

    For this reason I am currently working on 3 different YAML
    implementations.

    YAML
        The main YAML distribution will keeping evolving to support the
        entire YAML specification in pure Perl. This may not be the fastest
        or most stable module though. Currently, YAML.pm has lots of known
        bugs. It is mostly a great tool for dumping Perl data structures to
        a readable form.

    YAML::Tiny
        The point of YAML::Tiny is to strip YAML down to the 90% that people
        use most and offer that in a small, fast, stable, pure Perl form.
        YAML::Tiny will simply die when it is asked to do something it
        can't.

    YAML::Syck
        "libsyck" is the C based YAML processing library used by the Ruby
        programming language (and also Python, PHP and Pugs). YAML::Syck is
        the Perl binding to "libsyck". It should be very fast, but may have
        problems of its own. It will also require C compilation.

        NOTE: Audrey Tang has actually completed this module and it works
        great and is 10 times faster than YAML.pm.

    In the future, there will likely be even more YAML modules. Remember,
    people other than Ingy are allowed to write YAML modules!

FUNCTIONAL USAGE
    YAML is completely OO under the hood. Still it exports a few useful top
    level functions so that it is dead simple to use. These functions just
    do the OO stuff for you. If you want direct access to the OO API see the
    documentation for YAML::Dumper and YAML::Loader.

  Exported Functions
    The following functions are exported by YAML.pm by default. The reason
    they are exported is so that YAML works much like Data::Dumper. If you
    don't want functions to be imported, just use YAML with an empty import
    list:

        use YAML ();

    Dump(list-of-Perl-data-structures)
        Turn Perl data into YAML. This function works very much like
        Data::Dumper::Dumper(). It takes a list of Perl data strucures and
        dumps them into a serialized form. It returns a string containing
        the YAML stream. The structures can be references or plain scalars.

    Load(string-containing-a-YAML-stream)
        Turn YAML into Perl data. This is the opposite of Dump. Just like
        Storable's thaw() function or the eval() function in relation to
        Data::Dumper. It parses a string containing a valid YAML stream into
        a list of Perl data structures.

  Exportable Functions
    These functions are not exported by default but you can request them in
    an import list like this:

        use YAML qw'freeze thaw Bless';

    freeze() and thaw()
        Aliases to Dump() and Load() for Storable fans. This will also allow
        YAML.pm to be plugged directly into modules like POE.pm, that use
        the freeze/thaw API for internal serialization.

    DumpFile(filepath, list)
        Writes the YAML stream to a file instead of just returning a string.

    LoadFile(filepath)
        Reads the YAML stream from a file instead of a string.

    Bless(perl-node, [yaml-node | class-name])
        Associate a normal Perl node, with a yaml node. A yaml node is an
        object tied to the YAML::Node class. The second argument is either a
        yaml node that you've already created or a class (package) name that
        supports a yaml_dump() function. A yaml_dump() function should take
        a perl node and return a yaml node. If no second argument is
        provided, Bless will create a yaml node. This node is not returned,
        but can be retrieved with the Blessed() function.

        Here's an example of how to use Bless. Say you have a hash
        containing three keys, but you only want to dump two of them.
        Furthermore the keys must be dumped in a certain order. Here's how
        you do that:

            use YAML qw(Dump Bless);
            $hash = {apple => 'good', banana => 'bad', cauliflower => 'ugly'};
            print Dump $hash;
            Bless($hash)->keys(['banana', 'apple']);
            print Dump $hash;

        produces:

            ---
            apple: good
            banana: bad
            cauliflower: ugly
            ---
            banana: bad
            apple: good

        Bless returns the tied part of a yaml-node, so that you can call the
        YAML::Node methods. This is the same thing that YAML::Node::ynode()
        returns. So another way to do the above example is:

            use YAML qw(Dump Bless);
            use YAML::Node;
            $hash = {apple => 'good', banana => 'bad', cauliflower => 'ugly'};
            print Dump $hash;
            Bless($hash);
            $ynode = ynode(Blessed($hash));
            $ynode->keys(['banana', 'apple']);
            print Dump $hash;

        Note that Blessing a Perl data structure does not change it anyway.
        The extra information is stored separately and looked up by the
        Blessed node's memory address.

    Blessed(perl-node)
        Returns the yaml node that a particular perl node is associated with
        (see above). Returns undef if the node is not (YAML) Blessed.

GLOBAL OPTIONS
    YAML options are set using a group of global variables in the YAML
    namespace. This is similar to how Data::Dumper works.

    For example, to change the indentation width, do something like:

        local $YAML::Indent = 3;

    The current options are:

    DumperClass
        You can override which module/class YAML uses for Dumping data.

    LoaderClass
        You can override which module/class YAML uses for Loading data.

    Indent
        This is the number of space characters to use for each indentation
        level when doing a Dump(). The default is 2.

        By the way, YAML can use any number of characters for indentation at
        any level. So if you are editing YAML by hand feel free to do it
        anyway that looks pleasing to you; just be consistent for a given
        level.

    SortKeys
        Default is 1. (true)

        Tells YAML.pm whether or not to sort hash keys when storing a
        document.

        YAML::Node objects can have their own sort order, which is usually
        what you want. To override the YAML::Node order and sort the keys
        anyway, set SortKeys to 2.

    Stringify
        Default is 0. (false)

        Objects with string overloading should honor the overloading and
        dump the stringification of themselves, rather than the actual
        object's guts.

    UseHeader
        Default is 1. (true)

        This tells YAML.pm whether to use a separator string for a Dump
        operation. This only applies to the first document in a stream.
        Subsequent documents must have a YAML header by definition.

    UseVersion
        Default is 0. (false)

        Tells YAML.pm whether to include the YAML version on the
        separator/header.

            --- %YAML:1.0

    AnchorPrefix
        Default is ''.

        Anchor names are normally numeric. YAML.pm simply starts with '1'
        and increases by one for each new anchor. This option allows you to
        specify a string to be prepended to each anchor number.

    UseCode
        Setting the UseCode option is a shortcut to set both the DumpCode
        and LoadCode options at once. Setting UseCode to '1' tells YAML.pm
        to dump Perl code references as Perl (using B::Deparse) and to load
        them back into memory using eval(). The reason this has to be an
        option is that using eval() to parse untrusted code is, well,
        untrustworthy.

    DumpCode
        Determines if and how YAML.pm should serialize Perl code references.
        By default YAML.pm will dump code references as dummy placeholders
        (much like Data::Dumper). If DumpCode is set to '1' or 'deparse',
        code references will be dumped as actual Perl code.

        DumpCode can also be set to a subroutine reference so that you can
        write your own serializing routine. YAML.pm passes you the code ref.
        You pass back the serialization (as a string) and a format
        indicator. The format indicator is a simple string like: 'deparse'
        or 'bytecode'.

    LoadCode
        LoadCode is the opposite of DumpCode. It tells YAML if and how to
        deserialize code references. When set to '1' or 'deparse' it will
        use "eval()". Since this is potentially risky, only use this option
        if you know where your YAML has been.

        LoadCode can also be set to a subroutine reference so that you can
        write your own deserializing routine. YAML.pm passes the
        serialization (as a string) and a format indicator. You pass back
        the code reference.

    UseBlock
        YAML.pm uses heuristics to guess which scalar style is best for a
        given node. Sometimes you'll want all multiline scalars to use the
        'block' style. If so, set this option to 1.

        NOTE: YAML's block style is akin to Perl's here-document.

    UseFold
        If you want to force YAML to use the 'folded' style for all
        multiline scalars, then set $UseFold to 1.

        NOTE: YAML's folded style is akin to the way HTML folds text, except
        smarter.

    UseAliases
        YAML has an alias mechanism such that any given structure in memory
        gets serialized once. Any other references to that structure are
        serialized only as alias markers. This is how YAML can serialize
        duplicate and recursive structures.

        Sometimes, when you KNOW that your data is nonrecursive in nature,
        you may want to serialize such that every node is expressed in full.
        (ie as a copy of the original). Setting $YAML::UseAliases to 0 will
        allow you to do this. This also may result in faster processing
        because the lookup overhead is by bypassed.

        THIS OPTION CAN BE DANGEROUS. *If* your data is recursive, this
        option *will* cause Dump() to run in an endless loop, chewing up
        your computers memory. You have been warned.

    CompressSeries
        Default is 1.

        Compresses the formatting of arrays of hashes:

            -
              foo: bar
            - 
              bar: foo

        becomes:

            - foo: bar
            - bar: foo

        Since this output is usually more desirable, this option is turned
        on by default.

YAML TERMINOLOGY
    YAML is a full featured data serialization language, and thus has its
    own terminology.

    It is important to remember that although YAML is heavily influenced by
    Perl and Python, it is a language in its own right, not merely just a
    representation of Perl structures.

    YAML has three constructs that are conspicuously similar to Perl's hash,
    array, and scalar. They are called mapping, sequence, and string
    respectively. By default, they do what you would expect. But each
    instance may have an explicit or implicit tag (type) that makes it
    behave differently. In this manner, YAML can be extended to represent
    Perl's Glob or Python's tuple, or Ruby's Bigint.

    stream
        A YAML stream is the full sequence of unicode characters that a YAML
        parser would read or a YAML emitter would write. A stream may
        contain one or more YAML documents separated by YAML headers.

            ---
            a: mapping
            foo: bar
            ---
            - a
            - sequence

    document
        A YAML document is an independent data structure representation
        within a stream. It is a top level node. Each document in a YAML
        stream must begin with a YAML header line. Actually the header is
        optional on the first document.

            ---
            This: top level mapping
            is:
                - a
                - YAML
                - document

    header
        A YAML header is a line that begins a YAML document. It consists of
        three dashes, possibly followed by more info. Another purpose of the
        header line is that it serves as a place to put top level tag and
        anchor information.

            --- !recursive-sequence &001
            - * 001
            - * 001

    node
        A YAML node is the representation of a particular data stucture.
        Nodes may contain other nodes. (In Perl terms, nodes are like
        scalars. Strings, arrayrefs and hashrefs. But this refers to the
        serialized format, not the in-memory structure.)

    tag This is similar to a type. It indicates how a particular YAML node
        serialization should be transferred into or out of memory. For
        instance a Foo::Bar object would use the tag 'perl/Foo::Bar':

            - !perl/Foo::Bar
                foo: 42
                bar: stool

    collection
        A collection is the generic term for a YAML data grouping. YAML has
        two types of collections: mappings and sequences. (Similar to hashes
        and arrays)

    mapping
        A mapping is a YAML collection defined by unordered key/value pairs
        with unique keys. By default YAML mappings are loaded into Perl
        hashes.

            a mapping:
                foo: bar
                two: times two is 4

    sequence
        A sequence is a YAML collection defined by an ordered list of
        elements. By default YAML sequences are loaded into Perl arrays.

            a sequence:
                - one bourbon
                - one scotch
                - one beer

    scalar
        A scalar is a YAML node that is a single value. By default YAML
        scalars are loaded into Perl scalars.

            a scalar key: a scalar value

        YAML has many styles for representing scalars. This is important
        because varying data will have varying formatting requirements to
        retain the optimum human readability.

    plain scalar
        A plain sclar is unquoted. All plain scalars are automatic
        candidates for "implicit tagging". This means that their tag may be
        determined automatically by examination. The typical uses for this
        are plain alpha strings, integers, real numbers, dates, times and
        currency.

            - a plain string
            - -42
            - 3.1415
            - 12:34
            - 123 this is an error

    single quoted scalar
        This is similar to Perl's use of single quotes. It means no escaping
        except for single quotes which are escaped by using two adjacent
        single quotes.

            - 'When I say ''\n'' I mean "backslash en"'

    double quoted scalar
        This is similar to Perl's use of double quotes. Character escaping
        can be used.

            - "This scalar\nhas two lines, and a bell -->\a"

    folded scalar
        This is a multiline scalar which begins on the next line. It is
        indicated by a single right angle bracket. It is unescaped like the
        single quoted scalar. Line folding is also performed.

            - > 
             This is a multiline scalar which begins on
             the next line. It is indicated by a single
             carat. It is unescaped like the single
             quoted scalar. Line folding is also
             performed.

    block scalar
        This final multiline form is akin to Perl's here-document except
        that (as in all YAML data) scope is indicated by indentation.
        Therefore, no ending marker is required. The data is verbatim. No
        line folding.

            - |
                QTY  DESC          PRICE  TOTAL
                ---  ----          -----  -----
                  1  Foo Fighters  $19.95 $19.95
                  2  Bar Belles    $29.95 $59.90

    parser
        A YAML processor has four stages: parse, load, dump, emit.

        A parser parses a YAML stream. YAML.pm's Load() function contains a
        parser.

    loader
        The other half of the Load() function is a loader. This takes the
        information from the parser and loads it into a Perl data structure.

    dumper
        The Dump() function consists of a dumper and an emitter. The dumper
        walks through each Perl data structure and gives info to the
        emitter.

    emitter
        The emitter takes info from the dumper and turns it into a YAML
        stream.

        NOTE: In YAML.pm the parser/loader and the dumper/emitter code are
        currently very closely tied together. In the future they may be
        broken into separate stages.

    For more information please refer to the immensely helpful YAML
    specification available at <http://www.yaml.org/spec/>.

ysh - The YAML Shell
    The YAML distribution ships with a script called 'ysh', the YAML shell.
    ysh provides a simple, interactive way to play with YAML. If you type in
    Perl code, it displays the result in YAML. If you type in YAML it turns
    it into Perl code.

    To run ysh, (assuming you installed it along with YAML.pm) simply type:

        ysh [options]

    Please read the "ysh" documentation for the full details. There are lots
    of options.

BUGS & DEFICIENCIES
    If you find a bug in YAML, please try to recreate it in the YAML Shell
    with logging turned on ('ysh -L'). When you have successfully reproduced
    the bug, please mail the LOG file to the author (ingy@cpan.org).

    WARNING: This is still *ALPHA* code. Well, most of this code has been
    around for years...

    BIGGER WARNING: YAML.pm has been slow in the making, but I am committed
    to having top notch YAML tools in the Perl world. The YAML team is close
    to finalizing the YAML 1.1 spec. This version of YAML.pm is based off of
    a very old pre 1.0 spec. In actuality there isn't a ton of difference,
    and this YAML.pm is still fairly useful. Things will get much better in
    the future.

RESOURCES
    <http://lists.sourceforge.net/lists/listinfo/yaml-core> is the mailing
    list. This is where the language is discussed and designed.

    <http://www.yaml.org> is the official YAML website.

    <http://www.yaml.org/spec/> is the YAML 1.0 specification.

    <http://yaml.kwiki.org> is the official YAML wiki.

SEE ALSO
    See YAML::Syck. Fast!

AUTHOR
    Ingy döt Net <ingy@cpan.org>

    is resonsible for YAML.pm.

    The YAML serialization language is the result of years of collaboration
    between Oren Ben-Kiki, Clark Evans and Ingy döt Net. Several others have
    added help along the way.

COPYRIGHT
    Copyright (c) 2005, 2006, 2008. Ingy döt Net.

    Copyright (c) 2001, 2002, 2005. Brian Ingerson.

    Some parts copyright (c) 2009 - 2010 Adam Kennedy

    This program is free software; you can redistribute it and/or modify it
    under the same terms as Perl itself.

    See <http://www.perl.com/perl/misc/Artistic.html>

# yaml.el

[![MELPA](https://melpa.org/packages/yaml-badge.svg)](https://melpa.org/#/yaml)

yaml.el is a YAML parser written in Emacs List without any external
dependencies.  It provides an interface similar to the Emacs JSON
parsing utility.  The functions provided are as follows:

``` emacs-lisp
(yaml-parse-string string &rest args)
```

The following keyword args are accepted:

- `:object-type` specifies the Lisp data structure to store parsed
  objects data in.  It takes the following symbols:
  - `hash-table` (default)
  - `alist`
  - `plist`
- `:object-key-type` specifies how map keys should be handled. It takes the following symbols:
  - `string`
  - `symbol` (default) Use symbols as keys.  If `:object-type` is `plist`, this becomes the same as `keyword`.
  - `keyword` Always use keywords as keys.
- `:sequence-type` specifies the Lisp data structure to store the
  parsed sequences in.  It takes the following symbols:
  - `array` (default)
  - `list`
- `:null-object` specifies the lisp object to use for nulls.  Defaults
  to the symbol `:null`.
- `:false-object` specifies the lisp object to use for false.
  Defaults to the symbol `:false`.

```emacs-lisp
(yaml-encode object)
```

The function `yaml-encode` will encode a Lisp object to a YAML string.


## Installation

Until this is published to MELPA you will need to use the code from this repo directly.
You can put yaml.el in you load path directly or use a tool like use-package or straight.el.


## Examples

``` emacs-lisp
(require 'yaml)

(yaml-parse-string "
recipe:
  ingredients:
  - milk
  - eggs
  - oil
  - flour
  duration: 10
  steps: null" :object-type 'alist
               :sequence-type 'array
               :null-object :empty)

;; => (("recipe" ("ingredients" . ["milk" "eggs" "oil" "flour"]) ("duration" . 10) ("steps" . :empty)))

(yaml-parse-string "
translations:
  one: бір
  two: екі
  three: үш")

;; => #s(hash-table ... data ("translations" #s(hash-table ...)))


(yaml-encode '("omitted" ((count . 3) (value . 10) (items ("ruby" "diamond"))) "omitted"))

;; => "
- omitted
- count: 3
  value: 10
  items:
    ruby: [diamond]
- omitted"


```

## Caveats

Since this is implemented in Emacs Lisp performance is probably not the best.  An alternative implementation using libyaml exists and can be found [here](https://github.com/syohex/emacs-libyaml).

If you have a very deeply nested YAML file and your `max-lisp-eval-depth` variable is set too low, these is a chance that you might hit the maximum Lisp eval depth limit.  In the future I may work on changing the parsing algorithm to avoid this problem but in the meantime you can bump up the `max-lisp-eval-depth` variable in your config.

## Development

You can run the tests by executing

```
$ ~/path/to/git-repo/yaml.el
$ emacs -batch -l ert -l yaml.el -l yaml-tests.el -f ert-run-tests-batch-and-exit
```

#' Write a YAML file
#'
#' Write the YAML representation of an R object to a file
#'
#' If `file` is a non-open connection, an attempt is made to open it and
#' then close it after use.
#'
#' This function is a convenient wrapper around [as.yaml()].
#'
#' @param x The object to be converted.
#' @param file Either a character string naming a file or a [connection]
#'   open for writing.
#' @param fileEncoding Character string: if non-empty declares the encoding to
#'   be used on a file (not a connection) so the character data can be
#'   re-encoded as they are written. See [file()].
#' @param \dots Arguments to [as.yaml()].
#' @author Jeremy Stephens <jeremy.f.stephens@@vumc.org>
#' @seealso [as.yaml()], [read_yaml()],
#' [yaml.load_file()]
#' @export
#' @keywords data manip
#' @examples
#'
#' \dontrun{
#'   # writing to a file connection
#'   filename <- tempfile()
#'   con <- file(filename, "w")
#'   write_yaml(data.frame(a=1:10, b=letters[1:10], c=11:20), con)
#'   close(con)
#'
#'   # using a filename to specify output file
#'   write_yaml(data.frame(a=1:10, b=letters[1:10], c=11:20), filename)
#' }
#'
write_yaml <- function(x, file, fileEncoding = "UTF-8", ...) {
  result <- as.yaml(x, ...)

  if (is.character(file)) {
    file <-
      if (nzchar(fileEncoding)) {
        file(file, "w", encoding = fileEncoding)
      } else {
        file(file, "w")
      }
    on.exit(close(file))
  } else if (!isOpen(file, "w")) {
    open(file, "w")
    on.exit(close(file))
  }
  if (!inherits(file, "connection")) {
    stop("'file' must be a character string or connection")
  }

  cat(result, file = file, sep = "")
}

test_output_is_written_to_a_file_when_a_filename_is_specified <- function() {
  filename <- tempfile()
  write_yaml(1:3, filename)
  output <- readLines(filename)
  unlink(filename)
  checkEquals(c("- 1", "- 2", "- 3"), output)
}

test_that("output is written to a file when a filename is specified", {
  filename <- tempfile()
  write_yaml(1:3, filename)
  output <- readLines(filename)
  unlink(filename)
  expect_equal(output, c("- 1", "- 2", "- 3"))
})

#' @keywords internal
"_PACKAGE"

#' @useDynLib yaml, .registration = TRUE, .fixes = "C_"
NULL

## usethis namespace: start
## usethis namespace: end

## mockable bindings: start
## mockable bindings: end
NULL

#' @export
#' @rdname yaml.load
yaml.load_file <- function(input, error.label, readLines.warn = TRUE, ...) {
  if (missing(error.label)) {
    if (inherits(input, "connection")) {
      # try to guess filename
      s <- try(summary(input), silent = TRUE)
      if (
        !inherits(s, "try-error") && is.list(s) && "description" %in% names(s)
      ) {
        error.label <- s$description
      }
    } else if (is.character(input) && nzchar(input[1])) {
      error.label <- input[1]
    } else {
      error.label <- NULL
    }
  }

  if (is.character(input)) {
    con <- file(input, encoding = 'UTF-8')
    on.exit(close(con), add = TRUE)
  } else {
    con <- input
  }
  yaml.load(
    readLines(con, warn = readLines.warn),
    error.label = error.label,
    ...
  )
}

test_reading_from_a_connection_works <- function() {
  filename <- tempfile()
  cat("foo: 123", file=filename, sep="\n")
  foo <- file(filename, 'r')
  x <- yaml.load_file(foo)
  close(foo)
  unlink(filename)
  checkEquals(123L, x$foo)
}

test_reading_from_specified_filename_works <- function() {
  filename <- tempfile()
  cat("foo: 123", file=filename, sep="\n")
  x <- yaml.load_file(filename)
  unlink(filename)
  checkEquals(123L, x$foo)
}

test_reading_a_complicated_document_works <- function() {
  filename <- system.file(file.path("tests", "files", "test.yml"), package = "yaml")
  x <- yaml.load_file(filename)
  expected <- list(
    foo = list(one = 1, two = 2),
    bar = list(three = 3, four = 4),
    baz = list(list(one = 1, two = 2), list(three = 3, four = 4)),
    quux = list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6),
    corge = list(
      list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6),
      list(xyzzy = list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6))
    )
  )
  checkEquals(expected, x)
}

test_expressions_are_not_implicitly_converted_with_warning <- function() {
  warnings <- captureWarnings({
    filename <- tempfile()
    cat("!expr 123 + 456", file=filename, sep="\n")
    foo <- file(filename, 'r')
    x <- yaml.load_file(foo)
    close(foo)
    unlink(filename)
  })
  checkEquals("character", class(x))
  checkEquals("123 + 456", x)
  checkEquals("Evaluating R expressions (!expr) requires explicit `eval.expr=TRUE` option (see yaml.load help)", warnings)
}

test_expressions_are_explicitly_converted_without_warning <- function() {
  warnings <- captureWarnings({
    filename <- tempfile()
    cat("!expr 123 + 456", file=filename, sep="\n")
    foo <- file(filename, 'r')
    x <- yaml.load_file(foo, eval.expr = TRUE)
    close(foo)
    unlink(filename)
  })
  checkEquals("numeric", class(x))
  checkEquals(579, x)
  checkEquals(0, length(warnings))
}

test_expressions_are_unconverted <- function() {
  filename <- tempfile()
  cat("!expr 123 + 456", file=filename, sep="\n")
  foo <- file(filename, 'r')
  x <- yaml.load_file(foo, eval.expr = FALSE)
  close(foo)
  unlink(filename)

  checkEquals("character", class(x))
  checkEquals("123 + 456", x)
}

test_merge_specification_example_with_merge_override <- function() {
  filename <- system.file(file.path("tests", "files", "merge.yml"), package = "yaml")
  x <- yaml.load_file(filename, merge.precedence = "override")
  expected <- list(x = 1, y = 2, r = 10, label = "center/big")
  checkNamedListEquals(expected, x[[5]])
  checkNamedListEquals(expected, x[[6]])
  checkNamedListEquals(expected, x[[7]])
  checkNamedListEquals(expected, x[[8]])
}

check_named_list_equals <- function(x, y) {
  expect_equal(class(x), class(y))
  expect_equal(length(x), length(y))

  ns <- sort(names(x))
  expect_equal(ns, sort(names(y)))
  for (n in ns) {
    expect_equal(x[[n]], y[[n]])
  }
}

test_that("reading from a connection works", {
  filename <- tempfile()
  cat("foo: 123", file = filename, sep = "\n")
  foo <- file(filename, "r")
  x <- yaml.load_file(foo)
  close(foo)
  unlink(filename)
  expect_equal(x$foo, 123L)
})

test_that("reading from specified filename works", {
  filename <- tempfile()
  cat("foo: 123", file = filename, sep = "\n")
  x <- yaml.load_file(filename)
  unlink(filename)
  expect_equal(x$foo, 123L)
})

test_that("reading a complicated document works", {
  filename <- test_path("test.yml")
  x <- yaml.load_file(filename)
  expected <- list(
    foo = list(one = 1, two = 2),
    bar = list(three = 3, four = 4),
    baz = list(list(one = 1, two = 2), list(three = 3, four = 4)),
    quux = list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6),
    corge = list(
      list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6),
      list(
        xyzzy = list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6)
      )
    )
  )
  expect_equal(x, expected)
})

test_that("expressions are not implicitly converted with warning", {
  filename <- tempfile()
  cat("!expr 123 + 456", file = filename, sep = "\n")
  foo <- file(filename, "r")
  expect_warning(x <- yaml.load_file(foo), "Evaluating R expressions")
  close(foo)
  unlink(filename)
  expect_equal(class(x), "character")
  expect_equal(x, "123 + 456")
})

test_that("expressions are explicitly converted without warning", {
  filename <- tempfile()
  cat("!expr 123 + 456", file = filename, sep = "\n")
  foo <- file(filename, "r")
  expect_no_warning({
    x <- yaml.load_file(foo, eval.expr = TRUE)
  })
  close(foo)
  unlink(filename)
  expect_equal(class(x), "numeric")
  expect_equal(x, 579)
})

test_that("expressions are unconverted", {
  filename <- tempfile()
  cat("!expr 123 + 456", file = filename, sep = "\n")
  foo <- file(filename, "r")
  x <- yaml.load_file(foo, eval.expr = FALSE)
  close(foo)
  unlink(filename)

  expect_equal(class(x), "character")
  expect_equal(x, "123 + 456")
})

test_that("merge specification example with merge override", {
  filename <- test_path("merge.yml")
  x <- yaml.load_file(filename, merge.precedence = "override")
  expected <- list(x = 1, y = 2, r = 10, label = "center/big")
  check_named_list_equals(x[[5]], expected)
  check_named_list_equals(x[[6]], expected)
  check_named_list_equals(x[[7]], expected)
  check_named_list_equals(x[[8]], expected)
})

#' Alternative logical handler
#'
#' A yaml handler function that causes logical vectors to emit
#' `true`/`false` instead of `yes`/`no` values.
#'
#' Pass this function to [`as.yaml()`][as.yaml] as part of the
#' `handlers` argument list like `list(logical = verbatim_logical)`.
#'
#' @param x Logical vector to convert to `true`/`false`.
#' @return Returns a vector of strings of either `true` or `false` of
#' class `verbatim`.
#' @author Charles Dupont and James Goldie (jimjam-slam)
#' @seealso [as.yaml()]
#' @keywords data manip
#' @export
#' @examples
#'
#' vector <- c(TRUE, FALSE, TRUE)
#'
#' as.yaml(vector, handlers=list(logical=verbatim_logical))
#'
verbatim_logical <- function(x) {
  result <- tolower(as.logical(x))
  class(result) <- "verbatim"
  return(result)
}

#' Convert an R object into a YAML string
#'
#' If you set the `omap` option to TRUE, as.yaml will create ordered maps
#' (or omaps) instead of normal maps.
#'
#' The `column.major` option determines how a data frame is converted. If
#' TRUE, the data frame is converted into a map of sequences where the name of
#' each column is a key. If FALSE, the data frame is converted into a sequence
#' of maps, where each element in the sequence is a row.  You'll probably
#' almost always want to leave this as TRUE (which is the default), because
#' using [yaml.load()] on the resulting string returns an object
#' which is much more easily converted into a data frame via
#' [as.data.frame()].
#'
#' You can specify custom handler functions via the `handlers` argument.
#' This argument must be a named list of functions, where the names are R
#' object class names (i.e., 'numeric', 'data.frame', 'list', etc).  The
#' function(s) you provide will be passed one argument (the R object) and can
#' return any R object.  The returned object will be emitted normally.
#'
#' Character vectors that have a class of \sQuote{verbatim} will not be quoted
#' in the output YAML document except when the YAML specification requires it.
#' This means that you cannot do anything that would result in an invalid YAML
#' document, but you can emit strings that would otherwise be quoted.  This is
#' useful for changing how logical vectors are emitted (see below for example).
#'
#' Character vectors that have an attribute of \sQuote{quoted} will be wrapped
#' in double quotes (see below for example).
#'
#' You can specify YAML tags for R objects by setting the \sQuote{tag}
#' attribute to a character vector of length 1.  If you set a tag for a vector,
#' the tag will be applied to the YAML sequence as a whole, unless the vector
#' has only 1 element.  If you wish to tag individual elements, you must use a
#' list of 1-length vectors, each with a tag attribute.  Likewise, if you set a
#' tag for an object that would be emitted as a YAML mapping (like a data frame
#' or a named list), it will be applied to the mapping as a whole.  Tags can be
#' used in conjunction with YAML deserialization functions like
#' [yaml.load()] via custom handlers, however, if you set an internal
#' tag on an incompatible data type (like \dQuote{!seq 1.0}), errors will occur
#' when you try to deserialize the document.
#'
#' @param x The object to be converted.
#' @param line.sep The line separator character(s) to use.
#' @param indent The number of spaces to use for indenting.
#' @param omap Determines whether or not to convert a list to a YAML omap; see
#'   Details.
#' @param column.major Determines how to convert a data.frame; see Details.
#' @param unicode Determines whether or not to allow unescaped unicode
#'   characters in output.
#' @param precision Number of significant digits to use when formatting numeric
#'   values.
#' @param indent.mapping.sequence Determines whether or not to indent sequences
#'   in mapping context.
#' @param handlers Named list of custom handler functions for R objects; see
#'   Details.
#' @return Returns a YAML string which can be loaded using
#' [yaml.load()] or copied into a file for external use.
#' @author Jeremy Stephens <jeremy.f.stephens@@vumc.org>
#' @seealso [yaml.load()]
#' @references YAML: http://yaml.org
#'
#' YAML omap type: http://yaml.org/type/omap.html
#' @keywords data manip
#' @export
#' @examples
#'
#'   as.yaml(1:10)
#'   as.yaml(list(foo=1:10, bar=c("test1", "test2")))
#'   as.yaml(list(foo=1:10, bar=c("test1", "test2")), indent=3)
#'   as.yaml(list(foo=1:10, bar=c("test1", "test2")), indent.mapping.sequence=TRUE)
#'   as.yaml(data.frame(a=1:10, b=letters[1:10], c=11:20))
#'   as.yaml(list(a=1:2, b=3:4), omap=TRUE)
#'   as.yaml("multi\nline\nstring")
#'   as.yaml(function(x) x + 1)
#'   as.yaml(list(foo=list(list(x = 1, y = 2), list(x = 3, y = 4))))
#'
#'   # custom handler
#'   as.yaml(Sys.time(), handlers = list(
#'     POSIXct = function(x) format(x, "%Y-%m-%d")
#'   ))
#'
#'   # custom handler with verbatim output to change how logical vectors are
#'   # emitted
#'   as.yaml(c(TRUE, FALSE), handlers = list(
#'     logical = verbatim_logical))
#'
#'   # force quotes around a string
#'   port_def <- "80:80"
#'   attr(port_def, "quoted") <- TRUE
#'   x <- list(ports = list(port_def))
#'   as.yaml(x)
#'
#'   # custom tag for scalar
#'   x <- "thing"
#'   attr(x, "tag") <- "!thing"
#'   as.yaml(x)
#'
#'   # custom tag for sequence
#'   x <- 1:10
#'   attr(x, "tag") <- "!thing"
#'   as.yaml(x)
#'
#'   # custom tag for mapping
#'   x <- data.frame(a = letters[1:5], b = letters[6:10])
#'   attr(x, "tag") <- "!thing"
#'   as.yaml(x)
#'
#'   # custom tag for each element in a list
#'   x <- list(1, 2, 3)
#'   attr(x[[1]], "tag") <- "!a"
#'   attr(x[[2]], "tag") <- "!b"
#'   attr(x[[3]], "tag") <- "!c"
#'   as.yaml(x)
#'
as.yaml <- function(
  x,
  line.sep = c('\n', '\r\n', '\r'),
  indent = 2,
  omap = FALSE,
  column.major = TRUE,
  unicode = TRUE,
  precision = getOption('digits'),
  indent.mapping.sequence = FALSE,
  handlers = NULL
) {
  line.sep <- match.arg(line.sep)
  res <- .Call(
    C_serialize_to_yaml,
    x,
    line.sep,
    indent,
    omap,
    column.major,
    unicode,
    precision,
    indent.mapping.sequence,
    handlers,
    PACKAGE = "yaml"
  )
  Encoding(res) <- "UTF-8"
  res
}

test_named_list_is_converted <- function() {
  checkEquals("foo: bar\n", as.yaml(list(foo="bar")))

  x <- list(foo=1:10, bar=c("junk", "test"))
  y <- yaml.load(as.yaml(x))
  checkEquals(y$foo, x$foo)
  checkEquals(y$bar, x$bar)

  x <- list(foo=1:10, bar=list(foo=11:20, bar=letters[1:10]))
  y <- yaml.load(as.yaml(x))
  checkEquals(x$foo, y$foo)
  checkEquals(x$bar$foo, y$bar$foo)
  checkEquals(x$bar$bar, y$bar$bar)

  # nested lists
  x <- list(foo = list(a = 1, b = 2), bar = list(b = 4))
  y <- yaml.load(as.yaml(x))
  checkEquals(x$foo$a, y$foo$a)
  checkEquals(x$foo$b, y$foo$b)
  checkEquals(x$bar$b, y$bar$b)
}

test_data_frame_is_converted <- function() {
  x <- data.frame(a=1:10, b=letters[1:10], c=11:20)
  y <- as.data.frame(yaml.load(as.yaml(x)))
  checkEquals(x$a, y$a)
  checkEquals(x$b, y$b)
  checkEquals(x$c, y$c)

  x <- as.yaml(x, column.major = FALSE)
  y <- yaml.load(x)
  checkEquals(5L,  y[[5]]$a)
  checkEquals("e", y[[5]]$b)
  checkEquals(15L, y[[5]]$c)
}

test_empty_nested_list_is_converted <- function() {
  x <- list(foo=list())
  checkEquals("foo: []\n", as.yaml(x))
}

test_empty_nested_data_frame_is_converted <- function() {
  x <- list(foo=data.frame())
  checkEquals("foo: {}\n", as.yaml(x))
}

test_empty_nested_vector_is_converted <- function() {
  x <- list(foo=character())
  checkEquals("foo: []\n", as.yaml(x))
}

test_list_is_converted_as_omap <- function() {
  x <- list(a=1:2, b=3:4)
  expected <- "!!omap\n- a:\n  - 1\n  - 2\n- b:\n  - 3\n  - 4\n"
  checkEquals(expected, as.yaml(x, omap=TRUE))
}

test_nested_list_is_converted_as_omap <- function() {
  x <- list(a=list(c=list(e=1L, f=2L)), b=list(d=list(g=3L, h=4L)))
  expected <- "!!omap\n- a: !!omap\n  - c: !!omap\n    - e: 1\n    - f: 2\n- b: !!omap\n  - d: !!omap\n    - g: 3\n    - h: 4\n"
  checkEquals(expected, as.yaml(x, omap=TRUE))
}

test_omap_is_loaded <- function() {
  x <- yaml.load(as.yaml(list(a=1:2, b=3:4, c=5:6, d=7:8), omap=TRUE))
  checkEquals(c("a", "b", "c", "d"), names(x))
}

test_numeric_is_converted_correctly <- function() {
  result <- as.yaml(c(1, 5, 10, 15))
  checkEquals(result, "- 1.0\n- 5.0\n- 10.0\n- 15.0\n", label = result)
}

test_multiline_string_is_converted <- function() {
  checkEquals("|-\n  foo\n  bar\n", as.yaml("foo\nbar"))
  checkEquals("- foo\n- |-\n  bar\n  baz\n", as.yaml(c("foo", "bar\nbaz")))
  checkEquals("foo: |-\n  foo\n  bar\n", as.yaml(list(foo = "foo\nbar")))
  checkEquals("a:\n- foo\n- bar\n- |-\n  baz\n  quux\n", as.yaml(data.frame(a = c('foo', 'bar', 'baz\nquux'))))
}

test_function_is_converted <- function() {
  x <- function() { runif(100) }
  expected <- "!expr |\n  function ()\n  {\n      runif(100)\n  }\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_list_with_unnamed_items_is_converted <- function() {
  x <- list(foo=list(list(a = 1L, b = 2L), list(a = 3L, b = 4L)))
  expected <- "foo:\n- a: 1\n  b: 2\n- a: 3\n  b: 4\n"
  result <- as.yaml(x)
  checkEquals(result, expected)
}

test_pound_signs_are_escaped_in_strings <- function() {
  result <- as.yaml("foo # bar")
  checkEquals("'foo # bar'\n", result)
}

test_null_is_converted <- function() {
  checkEquals("~\n", as.yaml(NULL))
}

test_different_line_seps_are_used <- function() {
  result <- as.yaml(c('foo', 'bar'), line.sep = "\n")
  checkEquals("- foo\n- bar\n", result)

  result <- as.yaml(c('foo', 'bar'), line.sep = "\r\n")
  checkEquals("- foo\r\n- bar\r\n", result)

  result <- as.yaml(c('foo', 'bar'), line.sep = "\r")
  checkEquals("- foo\r- bar\r", result)
}

test_custom_indent_is_used <- function() {
  result <- as.yaml(list(foo=list(bar=list('foo', 'bar'))), indent = 3)
  checkEquals("foo:\n   bar:\n   - foo\n   - bar\n", result)
}

test_block_sequences_in_mapping_context_are_indented_when_option_is_true <- function() {
  result <- as.yaml(list(foo=list(bar=list('foo', 'bar'))), indent.mapping.sequence = TRUE)
  checkEquals("foo:\n  bar:\n    - foo\n    - bar\n", result)
}

test_indent_value_is_validated <- function() {
  checkException(as.yaml(list(foo=list(bar=list('foo', 'bar'))), indent = 0))
}

test_strings_are_escaped_properly <- function() {
  result <- as.yaml("12345")
  checkEquals("'12345'\n", result)
}

test_unicode_strings_are_not_escaped <- function() {
  # list('име' = 'Александар', 'презиме' = 'Благотић')
  a <- "\u0438\u043C\u0435" # name 1
  b <- "\u0410\u043B\u0435\u043A\u0441\u0430\u043D\u0434\u0430\u0440" # value 1
  c <- "\u043F\u0440\u0435\u0437\u0438\u043C\u0435" # name 2
  d <- "\u0411\u043B\u0430\u0433\u043E\u0442\u0438\u045B" # value 2
  x <- list(b, d)
  names(x) <- c(a, c)
  expected <- paste(a, ": ", b, "\n", c, ": ", d, "\n", sep="")
  result <- as.yaml(x, unicode = TRUE)
  checkEquals(expected, result, label = result)
}

test_unicode_strings_are_escaped <- function() {
  # 'é'
  x <- "\u00e9"
  result <- as.yaml(x, unicode = FALSE)
  checkEquals("\"\\xE9\"\n", result)
}

test_unicode_strings_are_not_escaped_by_default <- function() {
  # list('é')
  x <- list("\u00e9")
  result <- as.yaml(x)
  checkEquals("- \u00e9\n", result)
}

test_named_list_with_unicode_character_is_correct_converted <- function() {
  x <- list(special.char = "\u00e9")
  result <- as.yaml(x)
  checkEquals("special.char: \u00e9\n", result)
}

test_unknown_objects_cause_error <- function() {
  checkException(as.yaml(expression(foo <- bar)))
}

test_inf_is_emitted_properly <- function() {
  result <- as.yaml(Inf)
  checkEquals(".inf\n", result)
}

test_negative_inf_is_emitted_properly <- function() {
  result <- as.yaml(-Inf)
  checkEquals("-.inf\n", result)
}

test_nan_is_emitted_properly <- function() {
  result <- as.yaml(NaN)
  checkEquals(".nan\n", result)
}

test_logical_na_is_emitted_properly <- function() {
  result <- as.yaml(NA)
  checkEquals(".na\n", result)
}

test_numeric_na_is_emitted_properly <- function() {
  result <- as.yaml(NA_real_)
  checkEquals(".na.real\n", result)
}

test_integer_na_is_emitted_properly <- function() {
  result <- as.yaml(NA_integer_)
  checkEquals(".na.integer\n", result)
}

test_character_na_is_emitted_properly <- function() {
  result <- as.yaml(NA_character_)
  checkEquals(".na.character\n", result)
}

test_true_is_emitted_properly <- function() {
  result <- as.yaml(TRUE)
  checkEquals("yes\n", result)
}

test_false_is_emitted_properly <- function() {
  result <- as.yaml(FALSE)
  checkEquals("no\n", result)
}

test_named_list_keys_are_escaped_properly <- function() {
  result <- as.yaml(list(n = 123))
  checkEquals("'n': 123.0\n", result)
}

test_data_frame_keys_are_escaped_properly_when_row_major <- function() {
  result <- as.yaml(data.frame(n=1:3), column.major = FALSE)
  checkEquals("- 'n': 1\n- 'n': 2\n- 'n': 3\n", result)
}

test_scientific_notation_is_valid_yaml <- function() {
  result <- as.yaml(10000000)
  checkEquals("1.0e+07\n", result)
}

test_precision_must_be_in_the_range_1..22 <- function() {
  checkException(as.yaml(12345, precision = -1))
  checkException(as.yaml(12345, precision = 0))
  checkException(as.yaml(12345, precision = 23))
}

test_factor_with_missing_values_is_emitted_properly <- function() {
  x <- factor('foo', levels=c('bar', 'baz'))
  result <- as.yaml(x)
  checkEquals(".na.character\n", result)
}

test_very_small_negative_float_is_emitted_properly <- function() {
  result <- as.yaml(-7.62e-24)
  checkEquals("-7.62e-24\n", result)
}

test_very_small_positive_float_is_emitted_properly <- function() {
  result <- as.yaml(7.62e-24)
  checkEquals("7.62e-24\n", result)
}

test_numeric_zero_is_emitted_properly <- function() {
  result <- as.yaml(0.0)
  checkEquals("0.0\n", result)
}

test_numeric_negative_zero_is_emitted_properly <- function() {
  result <- as.yaml(-0.0)
  checkEquals("-0.0\n", result)
}

test_custom_handler_is_run_for_first_class <- function() {
  x <- "foo"
  class(x) <- "bar"
  result <- as.yaml(x, handlers = list(bar = function(x) paste0("x", x, "x")))
  checkEquals("xfoox\n", result)
}

test_custom_handler_is_run_for_second_class <- function() {
  x <- "foo"
  class(x) <- c("bar", "baz")
  result <- as.yaml(x, handlers = list(baz = function(x) paste0("x", x, "x")))
  checkEquals("xfoox\n", result)
}

test_custom_handler_with_verbatim_result <- function() {
  result <- as.yaml(TRUE, handlers = list(
    logical = function(x) {
      result <- ifelse(x, "true", "false")
      class(result) <- "verbatim"
      return(result)
    }
  ))
  checkEquals("true\n", result)
}

test_custom_handler_with_sequence_result <- function() {
  result <- as.yaml(c(1, 2, 3), handlers = list(
    numeric = function(x) {
      x + 1
    }
  ))
  checkEquals("- 2.0\n- 3.0\n- 4.0\n", result)
}

test_custom_handler_with_mapping_result <- function() {
  result <- as.yaml(1, handlers = list(
    numeric = function(x) {
      list(foo = 1:2, bar = 3:4)
    }
  ))
  checkEquals("foo:\n- 1\n- 2\nbar:\n- 3\n- 4\n", result)
}

test_custom_handler_with_function_result <- function() {
  result <- as.yaml(1, handlers = list(
    numeric = function(x) {
      function(y) y + 1
    }
  ))
  expected <- "!expr |\n  function (y)\n  y + 1\n"
  checkEquals(expected, result)
}

test_custom_tag_for_function <- function() {
  f <- function(x) x + 1
  attr(f, "tag") <- "!foo"
  expected <- "!foo |\n  function (x)\n  x + 1\n"
  result <- as.yaml(f)
  checkEquals(expected, result)
}

test_custom_tag_for_numeric_sequence <- function() {
  x <- c(1, 2, 3)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- 1.0\n- 2.0\n- 3.0\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_numeric_scalar <- function() {
  x <- 1
  attr(x, "tag") <- "!foo"
  expected <- "!foo 1.0\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_integer_sequence <- function() {
  x <- 1L:3L
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- 1\n- 2\n- 3\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_integer_scalar <- function() {
  x <- 1L
  attr(x, "tag") <- "!foo"
  expected <- "!foo 1\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_logical_sequence <- function() {
  x <- c(TRUE, FALSE)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- yes\n- no\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_logical_scalar <- function() {
  x <- TRUE
  attr(x, "tag") <- "!foo"
  expected <- "!foo yes\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_factor_sequence <- function() {
  x <- factor(c("foo", "bar"))
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- foo\n- bar\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_factor_scalar <- function() {
  x <- factor("foo")
  attr(x, "tag") <- "!foo"
  expected <- "!foo foo\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_character_sequence <- function() {
  x <- c("foo", "bar")
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- foo\n- bar\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_character_scalar <- function() {
  x <- "foo"
  attr(x, "tag") <- "!foo"
  expected <- "!foo foo\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_data_frame <- function() {
  x <- data.frame(a = 1:3, b = 4:6)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\na:\n- 1\n- 2\n- 3\nb:\n- 4\n- 5\n- 6\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_data_frame_column <- function() {
  x <- data.frame(a = 1:3, b = 4:6)
  attr(x$a, "tag") <- "!foo"
  expected <- "a: !foo\n- 1\n- 2\n- 3\nb:\n- 4\n- 5\n- 6\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_omap <- function() {
  x <- list(a=1:2, b=3:4)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- a:\n  - 1\n  - 2\n- b:\n  - 3\n  - 4\n"
  result <- as.yaml(x, omap = TRUE)
  checkEquals(expected, result)
}

test_custom_tag_for_named_list <- function() {
  x <- list(a=1:2, b=3:4)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\na:\n- 1\n- 2\nb:\n- 3\n- 4\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_custom_tag_for_unnamed_list <- function() {
  x <- list(1, 2, 3)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- 1.0\n- 2.0\n- 3.0\n"
  result <- as.yaml(x)
  checkEquals(expected, result)
}

test_quotes_for_string <- function() {
  port_def <- "80:80"
  attr(port_def, "quoted") <- TRUE
  x <- list(ports = list(port_def))
  result <- as.yaml(x)
  expected <- "ports:\n- \"80:80\"\n"
  checkEquals(expected, result)
}

test_no_dots_at_end <- function()
{
  result <- yaml::as.yaml(list(eol = "\n", a = 1), line.sep = "\n")
  checkEquals("eol: |2+\n\na: 1.0\n", result)
}

test_that("named list is converted", {
  expect_equal(as.yaml(list(foo = "bar")), "foo: bar\n")

  x <- list(foo = 1:10, bar = c("junk", "test"))
  y <- yaml.load(as.yaml(x))
  expect_equal(y$foo, x$foo)
  expect_equal(y$bar, x$bar)

  x <- list(foo = 1:10, bar = list(foo = 11:20, bar = letters[1:10]))
  y <- yaml.load(as.yaml(x))

  expect_equal(y$foo, x$foo)
  expect_equal(y$bar$foo, x$bar$foo)
  expect_equal(y$bar$bar, x$bar$bar)

  # nested lists
  x <- list(foo = list(a = 1, b = 2), bar = list(b = 4))
  y <- yaml.load(as.yaml(x))
  expect_equal(y$foo$a, x$foo$a)
  expect_equal(y$foo$b, x$foo$b)
  expect_equal(y$bar$b, x$bar$b)
})

test_that("data frame is converted", {
  x <- data.frame(a = 1:10, b = letters[1:10], c = 11:20)
  y <- as.data.frame(yaml.load(as.yaml(x)))
  expect_equal(y$a, x$a)

  expect_equal(y$b, x$b)
  expect_equal(y$c, x$c)

  x <- as.yaml(x, column.major = FALSE)
  y <- yaml.load(x)
  expect_equal(y[[5]]$a, 5L)
  expect_equal(y[[5]]$b, "e")
  expect_equal(y[[5]]$c, 15L)
})

test_that("empty nested list is converted", {
  x <- list(foo = list())
  expect_equal(as.yaml(x), "foo: []\n")
})

test_that("empty nested data frame is converted", {
  x <- list(foo = data.frame())
  expect_equal(as.yaml(x), "foo: {}\n")
})

test_that("empty nested vector is converted", {
  x <- list(foo = character())
  expect_equal(as.yaml(x), "foo: []\n")
})

test_that("list is converted as omap", {
  x <- list(a = 1:2, b = 3:4)
  expected <- "!!omap\n- a:\n  - 1\n  - 2\n- b:\n  - 3\n  - 4\n"
  expect_equal(as.yaml(x, omap = TRUE), expected)
})

test_that("nested list is converted as omap", {
  x <- list(
    a = list(c = list(e = 1L, f = 2L)),
    b = list(d = list(g = 3L, h = 4L))
  )
  expected <- "!!omap\n- a: !!omap\n  - c: !!omap\n    - e: 1\n    - f: 2\n- b: !!omap\n  - d: !!omap\n    - g: 3\n    - h: 4\n"
  expect_equal(as.yaml(x, omap = TRUE), expected)
})

test_that("omap is loaded", {
  x <- yaml.load(as.yaml(list(a = 1:2, b = 3:4, c = 5:6, d = 7:8), omap = TRUE))
  expect_equal(names(x), c("a", "b", "c", "d"))
})

test_that("numeric is converted correctly", {
  result <- as.yaml(c(1, 5, 10, 15))
  expect_equal(result, "- 1.0\n- 5.0\n- 10.0\n- 15.0\n")
})

test_that("multiline string is converted", {
  expect_equal(as.yaml("foo\nbar"), "|-\n  foo\n  bar\n")
  expect_equal(as.yaml(c("foo", "bar\nbaz")), "- foo\n- |-\n  bar\n  baz\n")
  expect_equal(as.yaml(list(foo = "foo\nbar")), "foo: |-\n  foo\n  bar\n")
  expect_equal(
    as.yaml(data.frame(a = c("foo", "bar", "baz\nquux"))),
    "a:\n- foo\n- bar\n- |-\n  baz\n  quux\n"
  )
})

test_that("function is converted", {
  x <- function() {
    runif(100)
  }
  expected <- "!expr |\n  function ()\n  {\n      runif(100)\n  }\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("list with unnamed items is converted", {
  x <- list(foo = list(list(a = 1L, b = 2L), list(a = 3L, b = 4L)))
  expected <- "foo:\n- a: 1\n  b: 2\n- a: 3\n  b: 4\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("pound signs are escaped in strings", {
  result <- as.yaml("foo # bar")
  expect_equal(result, "'foo # bar'\n")
})

test_that("null is converted", {
  expect_equal(as.yaml(NULL), "~\n")
})

test_that("different line seps are used", {
  result <- as.yaml(c("foo", "bar"), line.sep = "\n")
  expect_equal(result, "- foo\n- bar\n")

  result <- as.yaml(c("foo", "bar"), line.sep = "\r\n")
  expect_equal(result, "- foo\r\n- bar\r\n")

  result <- as.yaml(c("foo", "bar"), line.sep = "\r")
  expect_equal(result, "- foo\r- bar\r")
})

test_that("custom indent is used", {
  result <- as.yaml(list(foo = list(bar = list("foo", "bar"))), indent = 3)
  expect_equal(result, "foo:\n   bar:\n   - foo\n   - bar\n")
})

test_that("block sequences in mapping context are indented when option is true", {
  result <- as.yaml(
    list(foo = list(bar = list("foo", "bar"))),
    indent.mapping.sequence = TRUE
  )
  expect_equal(result, "foo:\n  bar:\n    - foo\n    - bar\n")
})

test_that("indent value is validated", {
  expect_error(as.yaml(list(foo = list(bar = list("foo", "bar"))), indent = 0))
})

test_that("strings are escaped properly", {
  result <- as.yaml("12345")
  expect_equal(result, "'12345'\n")
})

test_that("unicode strings are not escaped", {
  # list('име' = 'Александар', 'презиме' = 'Благотић')
  a <- "\u0438\u043C\u0435" # name 1
  b <- "\u0410\u043B\u0435\u043A\u0441\u0430\u043D\u0434\u0430\u0440" # value 1
  c <- "\u043F\u0440\u0435\u0437\u0438\u043C\u0435" # name 2
  d <- "\u0411\u043B\u0430\u0433\u043E\u0442\u0438\u045B" # value 2
  x <- list(b, d)
  names(x) <- c(a, c)
  expected <- paste(a, ": ", b, "\n", c, ": ", d, "\n", sep = "")
  result <- as.yaml(x, unicode = TRUE)
  expect_equal(result, expected)
})

test_that("unicode strings are escaped", {
  # 'é'
  x <- "\u00e9"
  result <- as.yaml(x, unicode = FALSE)
  expect_equal(result, "\"\\xE9\"\n")
})

test_that("unicode strings are not escaped by default", {
  # list('é')
  x <- list("\u00e9")
  result <- as.yaml(x)
  expect_equal(result, "- \u00e9\n")
})

test_that("named list with unicode character is correct converted", {
  x <- list(special.char = "\u00e9")
  result <- as.yaml(x)
  expect_equal(result, "special.char: \u00e9\n")
})

test_that("unknown objects cause error", {
  expect_error(as.yaml(expression(foo <- bar)))
})

test_that("inf is emitted properly", {
  result <- as.yaml(Inf)
  expect_equal(result, ".inf\n")
})

test_that("negative inf is emitted properly", {
  result <- as.yaml(-Inf)
  expect_equal(result, "-.inf\n")
})

test_that("nan is emitted properly", {
  result <- as.yaml(NaN)
  expect_equal(result, ".nan\n")
})

test_that("logical na is emitted properly", {
  result <- as.yaml(NA)
  expect_equal(result, ".na\n")
})

test_that("numeric na is emitted properly", {
  result <- as.yaml(NA_real_)
  expect_equal(result, ".na.real\n")
})

test_that("integer na is emitted properly", {
  result <- as.yaml(NA_integer_)
  expect_equal(result, ".na.integer\n")
})

test_that("character na is emitted properly", {
  result <- as.yaml(NA_character_)
  expect_equal(result, ".na.character\n")
})

test_that("true is emitted properly", {
  result <- as.yaml(TRUE)
  expect_equal(result, "yes\n")
})

test_that("false is emitted properly", {
  result <- as.yaml(FALSE)
  expect_equal(result, "no\n")
})

test_that("named list keys are escaped properly", {
  result <- as.yaml(list(n = 123))
  expect_equal(result, "'n': 123.0\n")
})

test_that("data frame keys are escaped properly when row major", {
  result <- as.yaml(data.frame(n = 1:3), column.major = FALSE)
  expect_equal(result, "- 'n': 1\n- 'n': 2\n- 'n': 3\n")
})

test_that("scientific notation is valid yaml", {
  result <- as.yaml(10000000)
  expect_equal(result, "1.0e+07\n")
})

test_that("precision must be in the range 1..22", {
  expect_error(as.yaml(12345, precision = -1))
  expect_error(as.yaml(12345, precision = 0))
  expect_error(as.yaml(12345, precision = 23))
})

test_that("factor with missing values is emitted properly", {
  x <- factor("foo", levels = c("bar", "baz"))
  result <- as.yaml(x)
  expect_equal(result, ".na.character\n")
})

test_that("very small negative float is emitted properly", {
  result <- as.yaml(-7.62e-24)
  expect_equal(result, "-7.62e-24\n")
})

test_that("very small positive float is emitted properly", {
  result <- as.yaml(7.62e-24)
  expect_equal(result, "7.62e-24\n")
})

test_that("numeric zero is emitted properly", {
  result <- as.yaml(0.0)
  expect_equal(result, "0.0\n")
})

test_that("numeric negative zero is emitted properly", {
  result <- as.yaml(-0.0)
  expect_equal(result, "-0.0\n")
})

test_that("custom handler is run for first class", {
  x <- "foo"
  class(x) <- "bar"
  result <- as.yaml(x, handlers = list(bar = function(x) paste0("x", x, "x")))
  expect_equal(result, "xfoox\n")
})

test_that("custom handler is run for second class", {
  x <- "foo"
  class(x) <- c("bar", "baz")
  result <- as.yaml(x, handlers = list(baz = function(x) paste0("x", x, "x")))
  expect_equal(result, "xfoox\n")
})

test_that("custom handler with verbatim result", {
  result <- as.yaml(
    TRUE,
    handlers = list(
      logical = function(x) {
        result <- ifelse(x, "true", "false")
        class(result) <- "verbatim"
        return(result)
      }
    )
  )
  expect_equal(result, "true\n")
})

test_that("custom handler with sequence result", {
  result <- as.yaml(
    c(1, 2, 3),
    handlers = list(
      numeric = function(x) {
        x + 1
      }
    )
  )
  expect_equal(result, "- 2.0\n- 3.0\n- 4.0\n")
})

test_that("custom handler with mapping result", {
  result <- as.yaml(
    1,
    handlers = list(
      numeric = function(x) {
        list(foo = 1:2, bar = 3:4)
      }
    )
  )
  expect_equal(result, "foo:\n- 1\n- 2\nbar:\n- 3\n- 4\n")
})

test_that("custom handler with function result", {
  result <- as.yaml(
    1,
    handlers = list(
      numeric = function(x) {
        function(y) y + 1
      }
    )
  )
  expected <- "!expr |\n  function (y)\n  y + 1\n"
  expect_equal(result, expected)
})

test_that("custom tag for function", {
  f <- function(x) x + 1
  attr(f, "tag") <- "!foo"
  expected <- "!foo |\n  function (x)\n  x + 1\n"
  result <- as.yaml(f)
  expect_equal(result, expected)
})

test_that("custom tag for numeric sequence", {
  x <- c(1, 2, 3)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- 1.0\n- 2.0\n- 3.0\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for numeric scalar", {
  x <- 1
  attr(x, "tag") <- "!foo"
  expected <- "!foo 1.0\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for integer sequence", {
  x <- 1L:3L
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- 1\n- 2\n- 3\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for integer scalar", {
  x <- 1L
  attr(x, "tag") <- "!foo"
  expected <- "!foo 1\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for logical sequence", {
  x <- c(TRUE, FALSE)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- yes\n- no\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for logical scalar", {
  x <- TRUE
  attr(x, "tag") <- "!foo"
  expected <- "!foo yes\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for factor sequence", {
  x <- factor(c("foo", "bar"))
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- foo\n- bar\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for factor scalar", {
  x <- factor("foo")
  attr(x, "tag") <- "!foo"
  expected <- "!foo foo\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for character sequence", {
  x <- c("foo", "bar")
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- foo\n- bar\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for character scalar", {
  x <- "foo"
  attr(x, "tag") <- "!foo"
  expected <- "!foo foo\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for data frame", {
  x <- data.frame(a = 1:3, b = 4:6)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\na:\n- 1\n- 2\n- 3\nb:\n- 4\n- 5\n- 6\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for data frame column", {
  x <- data.frame(a = 1:3, b = 4:6)
  attr(x$a, "tag") <- "!foo"
  expected <- "a: !foo\n- 1\n- 2\n- 3\nb:\n- 4\n- 5\n- 6\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for omap", {
  x <- list(a = 1:2, b = 3:4)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- a:\n  - 1\n  - 2\n- b:\n  - 3\n  - 4\n"
  result <- as.yaml(x, omap = TRUE)
  expect_equal(result, expected)
})

test_that("custom tag for named list", {
  x <- list(a = 1:2, b = 3:4)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\na:\n- 1\n- 2\nb:\n- 3\n- 4\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("custom tag for unnamed list", {
  x <- list(1, 2, 3)
  attr(x, "tag") <- "!foo"
  expected <- "!foo\n- 1.0\n- 2.0\n- 3.0\n"
  result <- as.yaml(x)
  expect_equal(result, expected)
})

test_that("quotes for string", {
  port_def <- "80:80"
  attr(port_def, "quoted") <- TRUE
  x <- list(ports = list(port_def))
  result <- as.yaml(x)
  expected <- "ports:\n- \"80:80\"\n"
  expect_equal(result, expected)
})

test_that("no dots at end", {
  result <- yaml::as.yaml(list(eol = "\n", a = 1), line.sep = "\n")
  expect_equal(result, "eol: |2+\n\na: 1.0\n")
})

#' Read a YAML file
#'
#' Read a YAML document from a file and create an R object from it
#'
#' This function is a convenient wrapper for [yaml.load()] and is a
#' nicer alternative to [yaml.load_file()].
#'
#' You can specify a label to be prepended to error messages via the
#' `error.label` argument.  If `error.label` is missing,
#' `read_yaml` will make an educated guess for the value of
#' `error.label` by either using the specified filename (when `file`
#' is a character vector) or using the description of the supplied connection
#' object (via the `summary` function).  If `text` is used, the
#' default value of `error.label` will be `NULL`.
#'
#' @param file Either a character string naming a file or a [connection]
#'   open for reading.
#' @param fileEncoding Character string: if non-empty declares the encoding
#'   used on a file (not a connection) so the character data can be re-encoded.
#'   See [file()].
#' @param text Character string: if `file` is not supplied and this is,
#'   then data are read from the value of `text` via a text connection.
#'   Notice that a literal string can be used to include (small) data sets
#'   within R code.
#' @param error.label A label to prepend to error messages (see Details).
#' @param readLines.warn Logical (default: TRUE). Suppress warnings from
#'   readLines used inside read_yaml.
#' @param ... Arguments to pass to [yaml.load()].
#' @return If the root YAML object is a map, a named list or list with an
#' attribute of 'keys' is returned.  If the root object is a sequence, a list
#' or vector is returned, depending on the contents of the sequence.  A vector
#' of length 1 is returned for single objects.
#' @author Jeremy Stephens <jeremy.f.stephens@@vumc.org>
#' @seealso [yaml.load()], [write_yaml()],
#' [yaml.load_file()]
#' @references YAML: http://yaml.org
#'
#' libyaml: https://pyyaml.org/wiki/LibYAML
#' @keywords programming data manip
#' @export
#' @examples
#'
#' \dontrun{
#'   # reading from a file connection
#'   filename <- tempfile()
#'   cat("test: data\n", file = filename)
#'   con <- file(filename, "r")
#'   read_yaml(con)
#'   close(con)
#'
#'   # using a filename to specify input file
#'   read_yaml(filename)
#' }
#'
#'   # reading from a character vector
#'   read_yaml(text="- hey\n- hi\n- hello")
#'
read_yaml <- function(
  file,
  fileEncoding = "UTF-8",
  text,
  error.label,
  readLines.warn = TRUE,
  ...
) {
  if (missing(file) && !missing(text)) {
    if (missing(error.label)) {
      error.label <- NULL
    }
    file <- textConnection(text, encoding = "UTF-8")
    on.exit(close(file))
  } else if (is.character(file)) {
    if (missing(error.label)) {
      error.label <- file
    }
    file <- if (nzchar(fileEncoding)) {
      file(file, "rt", encoding = fileEncoding)
    } else {
      file(file, "rt")
    }
    on.exit(close(file))
  } else if (inherits(file, "connection")) {
    if (missing(error.label)) {
      # try to guess filename
      s <- try(summary(file), silent = TRUE)
      if (
        !inherits(s, "try-error") && is.list(s) && "description" %in% names(s)
      ) {
        error.label <- s$description
      }
    }

    if (!isOpen(file, "rt")) {
      open(file, "rt")
      on.exit(close(file))
    }
  } else {
    stop("'file' must be a character string or connection")
  }

  string <- paste(readLines(file, warn = readLines.warn), collapse = "\n")
  yaml.load(string, error.label = error.label, ...)
}

test_reading_from_a_connection_works <- function() {
  filename <- tempfile()
  cat("foo: 123", file=filename, sep="\n")
  foo <- file(filename, 'r')
  x <- read_yaml(foo)
  close(foo)
  unlink(filename)
  checkEquals(123L, x$foo)
}

test_reading_from_specified_filename_works <- function() {
  filename <- tempfile()
  cat("foo: 123", file=filename, sep="\n")
  x <- read_yaml(filename)
  unlink(filename)
  checkEquals(123L, x$foo)
}

test_reading_from_text_works <- function() {
  x <- read_yaml(text="foo: 123")
  checkEquals(123L, x$foo)
}

test_reading_a_complicated_document_works <- function() {
  filename <- system.file(file.path("tests", "files", "test.yml"), package = "yaml")
  x <- read_yaml(filename)
  expected <- list(
    foo   = list(one=1, two=2),
    bar   = list(three=3, four=4),
    baz   = list(list(one=1, two=2), list(three=3, four=4)),
    quux  = list(one=1, two=2, three=3, four=4, five=5, six=6),
    corge = list(
      list(one=1, two=2, three=3, four=4, five=5, six=6),
      list(xyzzy=list(one=1, two=2, three=3, four=4, five=5, six=6))
    )
  )
  checkEquals(expected, x)
}

test_warns_about_incomplete_last_line <- function() 
{
  filename <- tempfile()
  cat("foo: 123", file=filename, sep="")
  foo <- file(filename, 'r')
  checkWarning(x <- read_yaml(foo))
  #, readLines.warn=FALSE)
  close(foo)
  unlink(filename)
  checkEquals(123L, x$foo)
}

test_supress_warning_incomplete_last_line <- function() 
{
  filename <- tempfile()
  cat("foo: 123", file=filename, sep="")
  foo <- file(filename, 'r')
  warnings <- captureWarnings(x <- read_yaml(foo, readLines.warn=FALSE))
  close(foo)
  unlink(filename)
  checkEquals(123L, x$foo)
  checkEquals(0, length(warnings))
}

test_that("reading from a connection works", {
  filename <- tempfile()
  cat("foo: 123", file = filename, sep = "\n")
  foo <- file(filename, "r")
  x <- read_yaml(foo)
  close(foo)
  unlink(filename)
  expect_equal(x$foo, 123L)
})

test_that("reading from specified filename works", {
  filename <- tempfile()
  cat("foo: 123", file = filename, sep = "\n")
  x <- read_yaml(filename)

  unlink(filename)
  expect_equal(x$foo, 123L)
})

test_that("reading from text works", {
  x <- read_yaml(text = "foo: 123")
  expect_equal(x$foo, 123L)
})

test_that("reading a complicated document works", {
  filename <- test_path("test.yml")
  x <- read_yaml(filename)
  expected <- list(
    foo = list(one = 1, two = 2),
    bar = list(three = 3, four = 4),
    baz = list(list(one = 1, two = 2), list(three = 3, four = 4)),
    quux = list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6),
    corge = list(
      list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6),
      list(
        xyzzy = list(one = 1, two = 2, three = 3, four = 4, five = 5, six = 6)
      )
    )
  )
  expect_equal(x, expected)
})

test_that("warns about incomplete last line", {
  filename <- tempfile()
  cat("foo: 123", file = filename, sep = "")
  foo <- file(filename, "r")
  expect_warning(x <- read_yaml(foo))
  close(foo)

  unlink(filename)
  expect_equal(x$foo, 123L)
})

test_that("supress warning incomplete last line", {
  filename <- tempfile()
  cat("foo: 123", file = filename, sep = "")
  foo <- file(filename, "r")
  expect_no_warning(x <- read_yaml(foo, readLines.warn = FALSE))
  close(foo)
  unlink(filename)
  expect_equal(x$foo, 123L)
})

#' Convert a YAML string into R objects
#'
#' Parse a YAML string and return R objects.
#'
#' Use `yaml.load` to load a YAML string.  For files and connections, use
#' `yaml.load_file`, which calls `yaml.load` with the contents of the
#' specified file or connection.
#'
#' Sequences of uniform data (e.g. a sequence of integers) are converted into
#' vectors.  If the sequence is not uniform, it's returned as a list. Maps are
#' converted into named lists by default, and all the keys in the map are
#' converted to strings.  If you don't want the keys to be coerced into
#' strings, set `as.named.list` to FALSE.  When it's FALSE, a list will be
#' returned with an additional attribute named 'keys', which is a list of the
#' un-coerced keys in the map (in the same order as the main list).
#'
#' You can specify custom handler functions via the `handlers` argument.
#' This argument must be a named list of functions, where the names are the
#' YAML types (i.e., 'int', 'float', 'seq', etc).  The functions you provide
#' will be passed one argument.  Custom handler functions for string types (all
#' types except sequence and map) will receive a character vector of length 1.
#' Custom sequence functions will be passed a list of objects.  Custom map
#' functions will be passed the object that the internal map handler creates,
#' which is either a named list or a list with a 'keys' attribute (depending on
#' `as.named.list`).  ALL functions you provide must return an object.
#' See the examples for custom handler use.
#'
#' You can specify a label to be prepended to error messages via the
#' `error.label` argument.  When using `yaml.load_file`, you can
#' either set the `error.label` argument explicitly or leave it missing.
#' If missing, `yaml.load_file` will make an educated guess for the value
#' of `error.label` by either using the specified filename (when
#' `input` is a character vector) or using the description of the supplied
#' connection object (via the `summary` function).  You can explicitly set
#' `error.label` to `NULL` if you don't want to use this
#' functionality.
#'
#' There is a built-in handler that will evaluate expressions that are tagged
#' with the \sQuote{!expr} tag.  Currently this handler is disabled by default
#' for security reasons.  If a \sQuote{!expr} tag exists and this is set to
#' FALSE a warning will occur. Alternately, you can set the option named
#' \sQuote{yaml.eval.expr} via the `options` function to turn on
#' evaluation.
#'
#' The `merge.precedence` parameter controls how merge keys are handled.
#' The YAML merge key specification is not specific about how key/value
#' conflicts are resolved during map merges.  As a result, various YAML library
#' implementations vary in merge key behavior (notably Python and Ruby).  This
#' package's default behavior (when `merge.precedence` is \sQuote{order})
#' is to give precedence to key/value pairs that appear first.  If you set
#' `merge.precedence` to \sQuote{override}, natural map key/value pairs
#' will override any duplicate keys found in merged maps, regardless of order.
#' This is the default behavior in Python's YAML library.
#'
#' This function uses the YAML parser provided by libyaml, which conforms to
#' the YAML 1.1 specification.
#'
#' @aliases yaml.load yaml.load_file
#' @param string The YAML string to be parsed.
#' @param as.named.list Whether or not to return a named list for maps (TRUE by
#'   default).
#' @param handlers Named list of custom handler functions for YAML types (see
#'   Details).
#' @param input A filename or connection; if `input` is a filename, that
#'   file must be encoded in UTF-8.
#' @param error.label A label to prepend to error messages (see Details).
#' @param eval.expr Whether or not to evaluate expressions found in the YAML
#'   document (see Details).
#' @param merge.precedence Precedence behavior during map merges (see
#'   Details).
#' @param merge.warning Whether or not to warn about ignored key/value pairs
#'   during map merges.
#' @param readLines.warn Logical (default: TRUE). Suppress warnings from
#'   readLines used inside read_yaml.
#' @param ... Arguments to pass to yaml.load.
#' @return If the root YAML object is a map, a named list or list with an
#' attribute of 'keys' is returned.  If the root object is a sequence, a list
#' or vector is returned, depending on the contents of the sequence.  A vector
#' of length 1 is returned for single objects.
#' @author Jeremy Stephens <jeremy.f.stephens@@vumc.org>
#' @seealso [as.yaml()]
#' @references YAML: http://yaml.org
#'
#' libyaml: https://pyyaml.org/wiki/LibYAML
#'
#' YAML merge specification: http://yaml.org/type/merge.html
#' @keywords programming data manip
#' @export
#' @examples
#'
#'   yaml.load("- hey\n- hi\n- hello")
#'   yaml.load("foo: 123\nbar: 456")
#'   yaml.load("- foo\n- bar\n- 3.14")
#'   yaml.load("foo: bar\n123: 456", as.named.list = FALSE)
#'
#' \dontrun{
#'   # reading from a file (uses readLines internally)
#'   filename <- tempfile()
#'   cat("foo: 123", file=filename, sep="\n")
#'   yaml.load_file(filename)
#' }
#'
#'   # custom scalar handler
#'   my.float.handler <- function(x) { as.numeric(x) + 123 }
#'   yaml.load("123.456", handlers=list("float#fix"=my.float.handler))
#'
#'   # custom sequence handler
#'   yaml.load("- 1\n- 2\n- 3", handlers=list(seq=function(x) { as.integer(x) + 3 }))
#'
#'   # custom map handler
#'   yaml.load("foo: 123", handlers=list(map=function(x) { x$foo <- x$foo + 123; x }))
#'
#'   # handling custom types
#'   yaml.load("!sqrt 555", handlers=list(sqrt=function(x) { sqrt(as.integer(x)) }))
#'   yaml.load("!foo\n- 1\n- 2", handlers=list(foo=function(x) { as.integer(x) + 1 }))
#'   yaml.load("!bar\none: 1\ntwo: 2", handlers=list(bar=function(x) { x$one <- "one"; x }))
#'
#'   # loading R expressions
#'   # NOTE: this will not be done by default in the near future
#'   doc <- yaml.load("inc: !expr function(x) x + 1", eval.expr=TRUE)
#'   doc$inc(1)
#'
#'   # adding a label to error messages
#'   try(yaml.load("*", error.label = "foo"))
#'
yaml.load <- function(
  string,
  as.named.list = TRUE,
  handlers = NULL,
  error.label = NULL,
  eval.expr = getOption("yaml.eval.expr", FALSE),
  merge.precedence = c("order", "override"),
  merge.warning = FALSE
) {
  string <- enc2utf8(paste(string, collapse = "\n"))
  eval.warning <- missing(eval.expr) && is.null(getOption("yaml.eval.expr"))
  merge.precedence <- match.arg(merge.precedence)

  .Call(
    C_unserialize_from_yaml,
    string,
    as.named.list,
    handlers,
    error.label,
    eval.expr,
    eval.warning,
    merge.precedence,
    merge.warning,
    PACKAGE = "yaml"
  )
}

test_named_list_is_not_returned <- function() {
  x <- yaml.load("hey: man\n123: 456\n", FALSE)
  checkEquals(2L, length(x))
  checkEquals(2L, length(attr(x, "keys")))

  x <- yaml.load("- dude\n- sup\n- 1.2345", FALSE)
  checkEquals(3L, length(x))
  checkEquals(0L, length(attr(x, "keys")))
  checkEquals("sup", x[[2]])

  x <- yaml.load("dude:\n  - 123\n  - sup", FALSE)
  checkEquals(1L, length(x))
  checkEquals(1L, length(attr(x, "keys")))
  checkEquals(2L, length(x[[1]]))
}

test_key_conflicts_throw_errors <- function() {
  checkException(yaml.load("hey: buddy\nhey: guy"));
}

test_named_list_is_returned <- function() {
  x <- yaml.load("hey: man\n123: 456\n", TRUE)
  checkEquals(2L, length(x))
  checkEquals(2L, length(names(x)))
  checkEquals(c("123", "hey"), sort(names(x)))
  checkEquals("man", x$hey)
}

test_uniform_sequences_are_coerced <- function() {
  x <- yaml.load("- 1\n- 2\n- 3")
  checkEquals(1:3, x)

  x <- yaml.load("- yes\n- no\n- yes")
  checkEquals(c(TRUE, FALSE, TRUE), x)

  x <- yaml.load("- 3.1\n- 3.14\n- 3.141")
  checkEquals(c(3.1, 3.14, 3.141), x)

  x <- yaml.load("- hey\n- hi\n- hello")
  checkEquals(c("hey", "hi", "hello"), x)
}

test_tag_type_conflicts_throws_error <- function() {
  checkException(yaml.load("!!str [1, 2, 3]"))
  checkException(yaml.load("!!str {foo: bar}"))
}

test_sequences_are_not_collapsed <- function() {
  x <- yaml.load("- [1, 2]\n- 3\n- [4, 5]")
  checkEquals(list(1:2, 3L, 4:5), x)
}

test_named_maps_are_merged_without_warnings <- function() {
  x <- yaml.load("foo: bar\n<<: {baz: boo}", TRUE)
  checkEquals(2L, length(x))
  checkEquals("bar", x$foo)
  checkEquals("boo", x$baz)

  expected <- list(foo = 'bar', quux = 'quux', baz = 'blah')
  warnings <- captureWarnings({
    x <- yaml.load("foo: bar\n<<: [{quux: quux}, {foo: doo}, {foo: junk}, {baz: blah}, {baz: boo}]", TRUE)
  })
  checkEquals(expected, x)
  checkEquals(0L, length(warnings))

  warnings <- captureWarnings({
    x <- yaml.load("foo: bar\n<<: {foo: baz}\n<<: {foo: quux}")
  })
  checkEquals(1L, length(x))
  checkEquals("bar", x$foo)
  checkEquals(0L, length(warnings))

  warnings <- captureWarnings({
    x <- yaml.load("<<: {foo: bar}\nfoo: baz")
  })
  checkEquals(list(foo = 'bar'), x)
  checkEquals(0L, length(warnings))
}

test_named_maps_are_merged_with_warnings <- function() {
  x <- yaml.load("foo: bar\n<<: {baz: boo}", as.named.list = TRUE, merge.warning = TRUE)
  checkEquals(2L, length(x))
  checkEquals("bar", x$foo)
  checkEquals("boo", x$baz)

  expected <- list(foo = 'bar', quux = 'quux', baz = 'blah')
  warnings <- captureWarnings({
    x <- yaml.load("foo: bar\n<<: [{quux: quux}, {foo: doo}, {foo: junk}, {baz: blah}, {baz: boo}]", as.named.list = TRUE, merge.warning = TRUE)
  })
  checkEquals(expected, x)
  checkEquals(c("Duplicate map key ignored during merge: 'foo'",
                 "Duplicate map key ignored during merge: 'foo'",
                 "Duplicate map key ignored during merge: 'baz'"), warnings)

  warnings <- captureWarnings({
    x <- yaml.load("foo: bar\n<<: {foo: baz}\n<<: {foo: quux}", as.named.list = TRUE, merge.warning = TRUE)
  })
  checkEquals(1L, length(x))
  checkEquals("bar", x$foo)
  checkEquals(c("Duplicate map key ignored during merge: 'foo'",
                 "Duplicate map key ignored during merge: 'foo'"), warnings)

  warnings <- captureWarnings({
    x <- yaml.load("<<: {foo: bar}\nfoo: baz", as.named.list = TRUE, merge.warning = TRUE)
  })
  checkEquals(list(foo = 'bar'), x)
  checkEquals(c("Duplicate map key ignored after merge: 'foo'"), warnings)
}

test_unnamed_maps_are_merged_without_warnings <- function() {
  x <- yaml.load("foo: bar\n<<: {baz: boo}", as.named.list = FALSE)
  checkEquals(2L, length(x))
  checkEquals(list("foo", "baz"), attr(x, 'keys'))
  checkEquals("bar", x[[1]])
  checkEquals("boo", x[[2]])

  warnings <- captureWarnings({
    x <- yaml.load("foo: bar\n<<: [{quux: quux}, {foo: doo}, {baz: boo}]", as.named.list = FALSE)
  })
  checkEquals(3L, length(x))
  checkEquals(list("foo", "quux", "baz"), attr(x, 'keys'))
  checkEquals("bar", x[[1]])
  checkEquals("quux", x[[2]])
  checkEquals("boo", x[[3]])
  checkEquals(0L, length(warnings))

  warnings <- captureWarnings({
    x <- yaml.load("<<: {foo: bar}\nfoo: baz", as.named.list = FALSE)
  })
  checkEquals(1L, length(x))
  checkEquals(list("foo"), attr(x, 'keys'))
  checkEquals("bar", x[[1]])
  checkEquals(0L, length(warnings))
}

test_unnamed_maps_are_merged_with_warnings <- function() {
  warnings <- captureWarnings({
    x <- yaml.load("foo: bar\n<<: {baz: boo}", as.named.list = FALSE, merge.warning = TRUE)
  })
  checkEquals(2L, length(x))
  checkEquals(list("foo", "baz"), attr(x, 'keys'))
  checkEquals("bar", x[[1]])
  checkEquals("boo", x[[2]])
  checkEquals(0L, length(warnings))

  warnings <- captureWarnings({
    x <- yaml.load("foo: bar\n<<: [{quux: quux}, {foo: doo}, {baz: boo}]", as.named.list = FALSE, merge.warning = TRUE)
  })
  checkEquals(3L, length(x))
  checkEquals(list("foo", "quux", "baz"), attr(x, 'keys'))
  checkEquals("bar", x[[1]])
  checkEquals("quux", x[[2]])
  checkEquals("boo", x[[3]])
  checkEquals("Duplicate map key ignored during merge: 'foo'", warnings)

  warnings <- captureWarnings({
    x <- yaml.load("<<: {foo: bar}\nfoo: baz", as.named.list = FALSE, merge.warning = TRUE)
  })
  checkEquals(1L, length(x))
  checkEquals(list("foo"), attr(x, 'keys'))
  checkEquals("bar", x[[1]])
  checkEquals(c("Duplicate map key ignored after merge: 'foo'"), warnings)
}

test_duplicate_keys_throws_an_error <- function() {
  checkException(yaml.load("foo: bar\nfoo: baz\n", TRUE))
}

test_duplicate_keys_with_merge_first_does_not_throw_an_error <- function() {
  result <- try(yaml.load("<<: {foo: bar}\nfoo: baz\n", TRUE))
  checkTrue(!inherits(result, "try-error"))
}

test_invalid_merges_throw_errors <- function() {
  checkException(yaml.load("foo: bar\n<<: [{leet: hax}, blargh, 123]", TRUE))
  checkException(yaml.load("foo: bar\n<<: [123, blargh, {leet: hax}]", TRUE))
  checkException(yaml.load("foo: bar\n<<: junk", TRUE))
}

test_syntax_errors_throw_errors <- function() {
  checkException(yaml.load("[whoa, this is some messed up]: yaml?!: dang"))
}

test_null_types_are_converted <- function() {
  x <- yaml.load("~")
  checkEquals(NULL, x)
}

#test_should_handle_binary_type <- function() {
#  x <- yaml.load("!!binary 0b101011")
#  checkEquals("0b101011", x)
#}

test_bool_yes_type_is_converted <- function() {
  x <- yaml.load("yes")
  checkEquals(TRUE, x)
}

test_bool_no_type_is_converted <- function() {
  x <- yaml.load("no")
  checkEquals(FALSE, x)
}

test_int_hex_type_is_converted <- function() {
  x <- yaml.load("0xF")
  checkEquals(15L, x)
}

test_int_oct_type_is_converted <- function() {
  x <- yaml.load("015")
  checkEquals(13L, x)
}

#test_should_handle_int_base60_type <- function() {
#  x <- yaml.load("1:20")
#  checkEquals("1:20", x)
#}

test_int_type_is_converted <- function() {
  x <- yaml.load("31337")
  checkEquals(31337L, x)
}

test_explicit_int_type_is_converted <- function() {
  x <- yaml.load("!!int 31337")
  checkEquals(31337L, x)
}

#test_should_handle_float_base60_type <- function() {
#  x <- yaml.load("1:20.5")
#  checkEquals("1:20.5", x)
#}

test_float_nan_type_is_converted <- function() {
  x <- yaml.load(".NaN")
  checkTrue(is.nan(x))
}

test_float_inf_type_is_converted <- function() {
  x <- yaml.load(".inf")
  checkEquals(Inf, x)
}

test_float_neginf_type_is_converted <- function() {
  x <- yaml.load("-.inf")
  checkEquals(-Inf, x)
}

test_float_type_is_converted <- function() {
  x <- yaml.load("123.456")
  checkEquals(123.456, x)
}

#test_should_handle_timestamp_iso8601_type <- function() {
#  x <- yaml.load("!timestamp#iso8601 2001-12-14t21:59:43.10-05:00")
#  checkEquals("2001-12-14t21:59:43.10-05:00", x)
#}

#test_should_handle_timestamp_spaced_type <- function() {
#  x <- yaml.load("!timestamp#spaced 2001-12-14 21:59:43.10 -5")
#  checkEquals("2001-12-14 21:59:43.10 -5", x)
#}

#test_should_handle_timestamp_ymd_type <- function() {
#  x <- yaml.load("!timestamp#ymd 2008-03-03")
#  checkEquals("2008-03-03", x)
#}

#test_should_handle_timestamp_type <- function() {
#  x <- yaml.load("!timestamp 2001-12-14t21:59:43.10-05:00")
#  checkEquals("2001-12-14t21:59:43.10-05:00", x)
#}

test_aliases_are_handled <- function() {
  x <- yaml.load("- &foo bar\n- *foo")
  checkEquals(c("bar", "bar"), x)
}

test_str_type_is_converted <- function() {
  x <- yaml.load("lickety split")
  checkEquals("lickety split", x)
}

test_bad_anchors_are_handled <- function() {
  warnings <- captureWarnings({
    x <- yaml.load("*blargh")
  })
  expected <- "_yaml.bad-anchor_"
  class(expected) <- "_yaml.bad-anchor_"
  checkEquals(expected, x)
  checkEquals("Unknown anchor: blargh", warnings)
}

test_custom_null_handler_is_applied <- function() {
  x <- yaml.load("~", handlers=list("null"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_binary_handler_is_applied <- function() {
  x <- yaml.load("!binary 0b101011", handlers=list("binary"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_bool_yes_handler_is_applied <- function() {
  x <- yaml.load("yes", handlers=list("bool#yes"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_bool_no_handler_is_applied <- function() {
  x <- yaml.load("no", handlers=list("bool#no"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_int_hex_handler_is_applied <- function() {
  x <- yaml.load("0xF", handlers=list("int#hex"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_int_oct_handler_is_applied <- function() {
  x <- yaml.load("015", handlers=list("int#oct"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_int_base60_is_not_coerced_by_default <- function() {
  x <- yaml.load("1:20")
  checkEquals("1:20", x)
}

test_custom_int_base60_handler_is_applied <- function() {
  x <- yaml.load("1:20", handlers=list("int#base60"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_int_handler_is_applied <- function() {
  x <- yaml.load("31337", handlers=list("int"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_float_base60_handler_is_applied <- function() {
  x <- yaml.load("1:20.5", handlers=list("float#base60"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_float_nan_handler_is_applied <- function() {
  x <- yaml.load(".NaN", handlers=list("float#nan"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_float_inf_handler_is_applied <- function() {
  x <- yaml.load(".inf", handlers=list("float#inf"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_float_neginf_handler_is_applied <- function() {
  x <- yaml.load("-.inf", handlers=list("float#neginf"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_float_handler_is_applied <- function() {
  x <- yaml.load("123.456", handlers=list("float#fix"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_timestamp_iso8601_handler_is_applied <- function() {
  x <- yaml.load("2001-12-14t21:59:43.10-05:00", handlers=list("timestamp#iso8601"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

#test_should_use_custom_timestamp_spaced_handler <- function() {
#  x <- yaml.load('!"timestamp#spaced" 2001-12-14 21:59:43.10 -5', handlers=list("timestamp#spaced"=function(x) { "argh!" }))
#  checkEquals("argh!", x)
#}

test_custom_timestamp_ymd_handler_is_applied <- function() {
  x <- yaml.load("2008-03-03", handlers=list("timestamp#ymd"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_custom_merge_handler_is_not_applied <- function() {
  warnings <- captureWarnings({
    x <- yaml.load("foo: &foo\n  bar: 123\n  baz: 456\n\njunk:\n  <<: *foo\n  bah: 789", handlers=list("merge"=function(x) { "argh!" }))
  })
  checkEquals(list(foo=list(bar=123, baz=456), junk=list(bar=123, baz=456, bah=789)), x)
  checkEquals("Custom handling for type 'merge' is not allowed; handler ignored", warnings)
}

test_custom_str_handler_is_applied <- function() {
  x <- yaml.load("lickety split", handlers=list("str"=function(x) { "argh!" }))
  checkEquals("argh!", x)
}

test_handler_for_unknown_type_is_applied <- function() {
  x <- yaml.load("!viking pillage", handlers=list(viking=function(x) { paste(x, "the village") }))
  checkEquals("pillage the village", x)
}

test_custom_seq_handler_is_applied <- function() {
  x <- yaml.load("- 1\n- 2\n- 3", handlers=list(seq=function(x) { as.integer(x) + 3L }))
  checkEquals(4:6, x)
}

test_custom_map_handler_is_applied <- function() {
  x <- yaml.load("foo: bar", handlers=list(map=function(x) { x$foo <- paste(x$foo, "yarr"); x }))
  checkEquals("bar yarr", x$foo)
}

test_custom_typed_seq_handler_is_applied <- function() {
  x <- yaml.load("!foo\n- 1\n- 2", handlers=list(foo=function(x) { as.integer(x) + 1L }))
  checkEquals(2:3, x)
}

test_custom_typed_map_handler_is_applied <- function() {
  x <- yaml.load("!foo\nuno: 1\ndos: 2", handlers=list(foo=function(x) { x$uno <- "uno"; x$dos <- "dos"; x }))
  checkEquals(list(uno="uno", dos="dos"), x)
}

# NOTE: this works, but R_tryEval doesn't return when called non-interactively
#test_should_handle_a_bad_handler <- function() {
#  x <- yaml.load("foo", handlers=list(str=function(x) { blargh }))
#  str(x)
#}

test_empty_documents_are_loaded <- function() {
  x <- yaml.load("")
  checkEquals(NULL, x)
  x <- yaml.load("# this document only has\n   # wickedly awesome comments")
  checkEquals(NULL, x)
}

test_omaps_are_loaded <- function() {
  x <- yaml.load("--- !omap\n- foo:\n  - 1\n  - 2\n- bar:\n  - 3\n  - 4")
  checkEquals(2L, length(x))
  checkEquals(c("foo", "bar"), names(x))
  checkEquals(1:2, x$foo)
  checkEquals(3:4, x$bar)
}

test_omaps_are_loaded_when_named_is_false <- function() {
  x <- yaml.load("--- !omap\n- 123:\n  - 1\n  - 2\n- bar:\n  - 3\n  - 4", FALSE)
  checkEquals(2L, length(x))
  checkEquals(list(123L, "bar"), attr(x, "keys"))
  checkEquals(1:2, x[[1]])
  checkEquals(3:4, x[[2]])
}

test_named_opam_with_duplicate_key_causes_error <- function() {
  checkException(yaml.load("--- !omap\n- foo:\n  - 1\n  - 2\n- foo:\n  - 3\n  - 4"))
}

test_unnamed_omap_with_duplicate_key_causes_error <- function() {
  checkException(yaml.load("--- !omap\n- foo:\n  - 1\n  - 2\n- foo:\n  - 3\n  - 4", FALSE))
}

test_invalid_omap_causes_error <- function() {
  checkException(yaml.load("--- !omap\nhey!"))
  checkException(yaml.load("--- !omap\n- sup?"))
}

test_expressions_are_not_implicitly_converted_with_warning <- function() {
  warnings <- captureWarnings({
    x <- yaml.load("!expr |\n  function() \n  {\n    'hey!'\n  }")
  })
  checkEquals("function() \n{\n  'hey!'\n}", x)
#  checkEquals("function", class(x))
#  checkEquals("hey!", x())
  checkEquals("Evaluating R expressions (!expr) requires explicit `eval.expr=TRUE` option (see yaml.load help)", warnings)
}

test_expressions_are_explicitly_converted_without_warning <- function() {
  warnings <- captureWarnings({
    x <- yaml.load("!expr |\n  function() \n  {\n    'hey!'\n  }", eval.expr = TRUE)
  })
  checkEquals("function", class(x))
  checkEquals("hey!", x())
  checkEquals(0, length(warnings))
}

test_expressions_are_explicitly_not_converted <- function() {
  x <- yaml.load("!expr 123 + 456", eval.expr = FALSE)
  checkEquals("123 + 456", x)
}

test_invalid_expressions_cause_error <- function() {
  checkException(yaml.load("!expr |\n  1+", eval.expr=TRUE))
}

# NOTE: this works, but R_tryEval doesn't return when called non-interactively
#test_should_error_for_expressions_with_eval_errors <- function() {
#  x <- try(yaml.load("!expr |\n  1 + non.existent.variable"))
#  assert(inherits(x, "try-error"))
#}

test_maps_are_in_ordered <- function() {
  x <- yaml.load("{a: 1, b: 2, c: 3}")
  checkEquals(c('a', 'b', 'c'), names(x))
}

test_illegal_recursive_anchor_is_handled <- function() {
  warnings <- captureWarnings({
    x <- yaml.load('&foo {foo: *foo}')
  })
  expected <- "_yaml.bad-anchor_"
  class(expected) <- "_yaml.bad-anchor_"
  checkEquals(expected, x$foo)
  checkEquals("Unknown anchor: foo", warnings)
}

test_dereferenced_aliases_have_unshared_names <- function() {
  x <- yaml.load('{foo: &foo {one: 1, two: 2}, bar: *foo}')
  x$foo$one <- 'uno'
  checkEquals(1L, x$bar$one)
}

test_multiple_anchors_are_handled <- function() {
  x <- yaml.load('{foo: &foo {one: 1}, bar: &bar {two: 2}, baz: *foo, quux: *bar}')
  expected <- list(
    foo = list(one = 1),
    bar = list(two = 2),
    baz = list(one = 1),
    quux = list(two = 2)
  )
  checkEquals(expected, x)
}

test_quoted_strings_are_preserved <- function() {
  x <- yaml.load("'12345'")
  checkEquals("12345", x)
}

test_inf_is_loaded_properly <- function() {
  result <- yaml.load(".inf\n...\n")
  checkEquals(Inf, result)
}

test_negative_inf_is_loaded_properly <- function() {
  result <- yaml.load("-.inf\n...\n")
  checkEquals(-Inf, result)
}

test_nan_is_loaded_properly <- function() {
  result <- yaml.load(".nan\n...\n")
  checkEquals(NaN, result)
}

test_logical_na_is_loaded_properly <- function() {
  result <- yaml.load(".na\n...\n")
  checkEquals(NA, result)
}

test_numeric_na_is_loaded_properly <- function() {
  result <- yaml.load(".na.real\n...\n")
  checkEquals(NA_real_, result)
}

test_integer_na_is_loaded_properly <- function() {
  result <- yaml.load(".na.integer\n...\n")
  checkEquals(NA_integer_, result)
}

test_character_na_is_loaded_properly <- function() {
  result <- yaml.load(".na.character\n...\n")
  checkEquals(NA_character_, result)
}

test_true_is_loaded_properly_from_y <- function() {
  result <- yaml.load("y\n...\n")
  checkEquals(TRUE, result)
}

test_false_is_loaded_properly_from_n <- function() {
  result <- yaml.load("n\n...\n")
  checkEquals(FALSE, result)
}

test_numeric_sequence_with_missing_values_loads_properly <- function() {
  result <- yaml.load("[1.2, 3.4, .na.real]")
  checkEquals(c(1.2, 3.4, NA), result)
}

test_integer_sequence_with_missing_values_loads_properly <- function() {
  result <- yaml.load("[1, 2, .na.integer]")
  checkEquals(c(1, 2, NA), result)
}

test_string_sequence_with_missing_values_loads_properly <- function() {
  result <- yaml.load("[foo, bar, .na.character]")
  checkEquals(c("foo", "bar", NA), result)
}

test_logical_sequence_with_missing_values_loads_properly <- function() {
  result <- yaml.load("[y, n, .na]")
  checkEquals(c(TRUE, FALSE, NA), result)
}

test_numeric_sequence_with_nans_loads_properly <- function() {
  result <- yaml.load("[1.2, 3.4, .nan]")
  checkEquals(c(1.2, 3.4, NaN), result)
}

test_numeric_represented_in_exponential_form_is_loaded_properly <- function() {
  checkEquals(1000000, yaml.load("1.0e+06"))
};

test_numeric_without_leading_digits_is_loaded_properly <- function() {
  checkEquals(0.9, yaml.load(".9"))
};

test_integer_overflow_creates_a_warning <- function() {
  checkWarning(result <- yaml.load("2147483648"))
  checkEquals(NA_integer_, result)
  checkWarning(result <- yaml.load("2147483649"))
  checkEquals(NA_integer_, result)
}

test_numeric_overflow_creates_a_warning <- function() {
  checkWarning(result <- yaml.load("1.797693e+309"))
  checkEquals(NA_real_, result)
}

test_list_of_one_list_is_loaded_properly <- function() {
  result <- yaml.load('a:\n -\n  - b\n  - c\n')
  checkEquals(list(a = list(c("b", "c"))), result)
}

test_override_merge_precedence <- function() {
  doc <- "[ &one { foo: bar }, { <<: *one, foo: baz } ]"
  expected <- list(list(foo = 'bar'), list(foo = 'baz'))
  result <- yaml.load(doc, merge.precedence = "override")
  checkEquals(expected, result)
}

test_explicit_bool_tag_for_true_value <- function() {
  doc <- "!!bool 'true'"
  expected <- TRUE
  result <- yaml.load(doc)
  checkEquals(expected, result)
}

test_explicit_bool_tag_for_false_value <- function() {
  doc <- "!!bool 'false'"
  expected <- FALSE
  result <- yaml.load(doc)
  checkEquals(expected, result)
}

test_explicit_bool_tag_for_na_value <- function() {
  doc <- "!!bool '.na'"
  expected <- NA
  result <- yaml.load(doc)
  checkEquals(expected, result)
}

test_explicit_bool_tag_for_invalid_value <- function() {
  doc <- "!!bool foo"
  expected <- NA
  warnings <- captureWarnings({
    result <- yaml.load(doc)
  })
  checkEquals(expected, result)
  checkEquals(c("NAs introduced by coercion: foo is not a recognized boolean value"), warnings)
}

test_builtin_as_handler_works <- function() {
  x <- "{a: 1, b: 2, c: 3}"
  warnings <- captureWarnings({
    results <- yaml.load(x, handlers=list(int=as.double))
  })
  checkEquals(class(results$a), "numeric")
  checkEquals(0, length(warnings))
}

test_that("named list is not returned", {
  x <- yaml.load("hey: man\n123: 456\n", FALSE)
  expect_equal(length(x), 2L)
  expect_equal(length(attr(x, "keys")), 2L)

  x <- yaml.load("- dude\n- sup\n- 1.2345", FALSE)
  expect_equal(length(x), 3L)

  expect_equal(length(attr(x, "keys")), 0L)
  expect_equal(x[[2]], "sup")

  x <- yaml.load("dude:\n  - 123\n  - sup", FALSE)
  expect_equal(length(x), 1L)
  expect_equal(length(attr(x, "keys")), 1L)
  expect_equal(length(x[[1]]), 2L)
})

test_that("key conflicts throw errors", {
  expect_error(yaml.load("hey: buddy\nhey: guy"))
})

test_that("named list is returned", {
  x <- yaml.load("hey: man\n123: 456\n", TRUE)
  expect_equal(length(x), 2L)
  expect_equal(length(names(x)), 2L)
  expect_equal(sort(names(x)), c("123", "hey"))
  expect_equal(x$hey, "man")
})

test_that("uniform sequences are coerced", {
  x <- yaml.load("- 1\n- 2\n- 3")
  expect_equal(x, 1:3)

  x <- yaml.load("- yes\n- no\n- yes")
  expect_equal(x, c(TRUE, FALSE, TRUE))

  x <- yaml.load("- 3.1\n- 3.14\n- 3.141")
  expect_equal(x, c(3.1, 3.14, 3.141))

  x <- yaml.load("- hey\n- hi\n- hello")
  expect_equal(x, c("hey", "hi", "hello"))
})

test_that("tag type conflicts throws error", {
  expect_error(yaml.load("!!str [1, 2, 3]"))
  expect_error(yaml.load("!!str {foo: bar}"))
})

test_that("sequences are not collapsed", {
  x <- yaml.load("- [1, 2]\n- 3\n- [4, 5]")
  expect_equal(x, list(1:2, 3L, 4:5))
})

test_that("named maps are merged without warnings", {
  x <- yaml.load("foo: bar\n<<: {baz: boo}", TRUE)
  expect_equal(length(x), 2L)
  expect_equal(x$foo, "bar")
  expect_equal(x$baz, "boo")

  expected <- list(foo = "bar", quux = "quux", baz = "blah")
  expect_no_warning({
    x <- yaml.load(
      "foo: bar\n<<: [{quux: quux}, {foo: doo}, {foo: junk}, {baz: blah}, {baz: boo}]",
      TRUE
    )
  })
  expect_equal(x, expected)

  expect_no_warning({
    x <- yaml.load("foo: bar\n<<: {foo: baz}\n<<: {foo: quux}")
  })
  expect_equal(length(x), 1L)
  expect_equal(x$foo, "bar")

  expect_no_warning({
    x <- yaml.load("<<: {foo: bar}\nfoo: baz")
  })
  expect_equal(x, list(foo = "bar"))
})

test_that("named maps are merged with warnings", {
  x <- yaml.load(
    "foo: bar\n<<: {baz: boo}",
    as.named.list = TRUE,
    merge.warning = TRUE
  )
  expect_equal(length(x), 2L)
  expect_equal(x$foo, "bar")
  expect_equal(x$baz, "boo")

  expected <- list(foo = "bar", quux = "quux", baz = "blah")
  expect_warning(
    expect_warning(
      expect_warning(
        x <- yaml.load(
          "foo: bar\n<<: [{quux: quux}, {foo: doo}, {foo: junk}, {baz: blah}, {baz: boo}]",
          as.named.list = TRUE,
          merge.warning = TRUE
        ),
        "Duplicate map key ignored during merge: 'foo'"
      ),
      "Duplicate map key ignored during merge: 'foo'"
    ),
    "Duplicate map key ignored during merge: 'baz'"
  )
  expect_equal(x, expected)

  expect_warning(
    expect_warning(
      x <- yaml.load(
        "foo: bar\n<<: {foo: baz}\n<<: {foo: quux}",
        as.named.list = TRUE,
        merge.warning = TRUE
      ),
      "Duplicate map key ignored during merge: 'foo'"
    ),
    "Duplicate map key ignored during merge: 'foo'"
  )
  expect_equal(length(x), 1L)
  expect_equal(x$foo, "bar")

  expect_warning(
    x <- yaml.load(
      "<<: {foo: bar}\nfoo: baz",
      as.named.list = TRUE,
      merge.warning = TRUE
    ),
    "Duplicate map key ignored after merge: 'foo'"
  )
  expect_equal(x, list(foo = "bar"))
})

test_that("unnamed maps are merged without warnings", {
  x <- yaml.load("foo: bar\n<<: {baz: boo}", as.named.list = FALSE)
  expect_equal(length(x), 2L)
  expect_equal(attr(x, "keys"), list("foo", "baz"))
  expect_equal(x[[1]], "bar")
  expect_equal(x[[2]], "boo")

  expect_no_warning({
    x <- yaml.load(
      "foo: bar\n<<: [{quux: quux}, {foo: doo}, {baz: boo}]",
      as.named.list = FALSE
    )
  })
  expect_equal(length(x), 3L)
  expect_equal(attr(x, "keys"), list("foo", "quux", "baz"))
  expect_equal(x[[1]], "bar")
  expect_equal(x[[2]], "quux")
  expect_equal(x[[3]], "boo")

  expect_no_warning({
    x <- yaml.load("<<: {foo: bar}\nfoo: baz", as.named.list = FALSE)
  })
  expect_equal(length(x), 1L)
  expect_equal(attr(x, "keys"), list("foo"))
  expect_equal(x[[1]], "bar")
})

test_that("unnamed maps are merged with warnings", {
  expect_no_warning({
    x <- yaml.load(
      "foo: bar\n<<: {baz: boo}",
      as.named.list = FALSE,
      merge.warning = TRUE
    )
  })
  expect_equal(length(x), 2L)
  expect_equal(attr(x, "keys"), list("foo", "baz"))
  expect_equal(x[[1]], "bar")
  expect_equal(x[[2]], "boo")

  expect_warning(
    x <- yaml.load(
      "foo: bar\n<<: [{quux: quux}, {foo: doo}, {baz: boo}]",
      as.named.list = FALSE,
      merge.warning = TRUE
    ),
    "Duplicate map key ignored during merge: 'foo'"
  )
  expect_equal(length(x), 3L)
  expect_equal(attr(x, "keys"), list("foo", "quux", "baz"))
  expect_equal(x[[1]], "bar")
  expect_equal(x[[2]], "quux")
  expect_equal(x[[3]], "boo")

  expect_warning(
    x <- yaml.load(
      "<<: {foo: bar}\nfoo: baz",
      as.named.list = FALSE,
      merge.warning = TRUE
    ),
    "Duplicate map key ignored after merge: 'foo'"
  )
  expect_equal(length(x), 1L)
  expect_equal(attr(x, "keys"), list("foo"))
  expect_equal(x[[1]], "bar")
})

test_that("duplicate keys throws an error", {
  expect_error(yaml.load("foo: bar\nfoo: baz\n", TRUE))
})

test_that("duplicate keys with merge first does not throw an error", {
  result <- try(yaml.load("<<: {foo: bar}\nfoo: baz\n", TRUE), silent = TRUE)
  expect_false(inherits(result, "try-error"))
})

test_that("invalid merges throw errors", {
  expect_error(yaml.load("foo: bar\n<<: [{leet: hax}, blargh, 123]", TRUE))
  expect_error(yaml.load("foo: bar\n<<: [123, blargh, {leet: hax}]", TRUE))
  expect_error(yaml.load("foo: bar\n<<: junk", TRUE))
})

test_that("syntax errors throw errors", {
  expect_error(yaml.load("[whoa, this is some messed up]: yaml?!: dang"))
})

test_that("null types are converted", {
  x <- yaml.load("~")
  expect_null(x)
})

test_that("bool yes type is converted", {
  x <- yaml.load("yes")
  expect_equal(x, TRUE)
})

test_that("bool no type is converted", {
  x <- yaml.load("no")
  expect_equal(x, FALSE)
})

test_that("int hex type is converted", {
  x <- yaml.load("0xF")
  expect_equal(x, 15L)
})

test_that("int oct type is converted", {
  x <- yaml.load("015")
  expect_equal(x, 13L)
})

test_that("int type is converted", {
  x <- yaml.load("31337")
  expect_equal(x, 31337L)
})

test_that("explicit int type is converted", {
  x <- yaml.load("!!int 31337")
  expect_equal(x, 31337L)
})

test_that("float nan type is converted", {
  x <- yaml.load(".NaN")
  expect_true(is.nan(x))
})

test_that("float inf type is converted", {
  x <- yaml.load(".inf")
  expect_equal(x, Inf)
})

test_that("float neginf type is converted", {
  x <- yaml.load("-.inf")
  expect_equal(x, -Inf)
})

test_that("float type is converted", {
  x <- yaml.load("123.456")
  expect_equal(x, 123.456)
})

test_that("aliases are handled", {
  x <- yaml.load("- &foo bar\n- *foo")
  expect_equal(x, c("bar", "bar"))
})

test_that("str type is converted", {
  x <- yaml.load("lickety split")
  expect_equal(x, "lickety split")
})

test_that("bad anchors are handled", {
  expect_warning(x <- yaml.load("*blargh"), "Unknown anchor: blargh")
  expected <- "_yaml.bad-anchor_"
  class(expected) <- "_yaml.bad-anchor_"
  expect_equal(x, expected)
})

test_that("custom null handler is applied", {
  x <- yaml.load(
    "~",
    handlers = list("null" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom binary handler is applied", {
  x <- yaml.load(
    "!binary 0b101011",
    handlers = list("binary" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom bool yes handler is applied", {
  x <- yaml.load(
    "yes",
    handlers = list("bool#yes" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom bool no handler is applied", {
  x <- yaml.load(
    "no",
    handlers = list("bool#no" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom int hex handler is applied", {
  x <- yaml.load(
    "0xF",
    handlers = list("int#hex" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom int oct handler is applied", {
  x <- yaml.load(
    "015",
    handlers = list("int#oct" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("int base60 is not coerced by default", {
  x <- yaml.load("1:20")
  expect_equal(x, "1:20")
})

test_that("custom int base60 handler is applied", {
  x <- yaml.load(
    "1:20",
    handlers = list("int#base60" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom int handler is applied", {
  x <- yaml.load(
    "31337",
    handlers = list("int" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom float base60 handler is applied", {
  x <- yaml.load(
    "1:20.5",
    handlers = list("float#base60" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom float nan handler is applied", {
  x <- yaml.load(
    ".NaN",
    handlers = list("float#nan" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom float inf handler is applied", {
  x <- yaml.load(
    ".inf",
    handlers = list("float#inf" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom float neginf handler is applied", {
  x <- yaml.load(
    "-.inf",
    handlers = list("float#neginf" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom float handler is applied", {
  x <- yaml.load(
    "123.456",
    handlers = list("float#fix" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom timestamp iso8601 handler is applied", {
  x <- yaml.load(
    "2001-12-14t21:59:43.10-05:00",
    handlers = list("timestamp#iso8601" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom timestamp ymd handler is applied", {
  x <- yaml.load(
    "2008-03-03",
    handlers = list("timestamp#ymd" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("custom merge handler is not applied", {
  expect_warning(
    x <- yaml.load(
      "foo: &foo\n  bar: 123\n  baz: 456\n\njunk:\n  <<: *foo\n  bah: 789",
      handlers = list("merge" = function(x) {
        "argh!"
      })
    ),
    "Custom handling for type 'merge' is not allowed; handler ignored"
  )
  expect_equal(
    x,
    list(
      foo = list(bar = 123, baz = 456),
      junk = list(bar = 123, baz = 456, bah = 789)
    )
  )
})

test_that("custom str handler is applied", {
  x <- yaml.load(
    "lickety split",
    handlers = list("str" = function(x) {
      "argh!"
    })
  )
  expect_equal(x, "argh!")
})

test_that("handler for unknown type is applied", {
  x <- yaml.load(
    "!viking pillage",
    handlers = list(viking = function(x) {
      paste(x, "the village")
    })
  )
  expect_equal(x, "pillage the village")
})

test_that("custom seq handler is applied", {
  x <- yaml.load(
    "- 1\n- 2\n- 3",
    handlers = list(seq = function(x) {
      as.integer(x) + 3L
    })
  )
  expect_equal(x, 4:6)
})

test_that("custom map handler is applied", {
  x <- yaml.load(
    "foo: bar",
    handlers = list(map = function(x) {
      x$foo <- paste(x$foo, "yarr")
      x
    })
  )
  expect_equal(x$foo, "bar yarr")
})

test_that("custom typed seq handler is applied", {
  x <- yaml.load(
    "!foo\n- 1\n- 2",
    handlers = list(foo = function(x) {
      as.integer(x) + 1L
    })
  )
  expect_equal(x, 2:3)
})

test_that("custom typed map handler is applied", {
  x <- yaml.load(
    "!foo\nuno: 1\ndos: 2",
    handlers = list(foo = function(x) {
      x$uno <- "uno"
      x$dos <- "dos"
      x
    })
  )
  expect_equal(x, list(uno = "uno", dos = "dos"))
})

test_that("empty documents are loaded", {
  x <- yaml.load("")
  expect_null(x)
  x <- yaml.load("# this document only has\n   # wickedly awesome comments")
  expect_null(x)
})

test_that("omaps are loaded", {
  x <- yaml.load("--- !omap\n- foo:\n  - 1\n  - 2\n- bar:\n  - 3\n  - 4")
  expect_equal(length(x), 2L)
  expect_equal(names(x), c("foo", "bar"))
  expect_equal(x$foo, 1:2)
  expect_equal(x$bar, 3:4)
})

test_that("omaps are loaded when named is false", {
  x <- yaml.load("--- !omap\n- 123:\n  - 1\n  - 2\n- bar:\n  - 3\n  - 4", FALSE)
  expect_equal(length(x), 2L)
  expect_equal(attr(x, "keys"), list(123L, "bar"))
  expect_equal(x[[1]], 1:2)
  expect_equal(x[[2]], 3:4)
})

test_that("named opam with duplicate key causes error", {
  expect_error(yaml.load(
    "--- !omap\n- foo:\n  - 1\n  - 2\n- foo:\n  - 3\n  - 4"
  ))
})

test_that("unnamed omap with duplicate key causes error", {
  expect_error(yaml.load(
    "--- !omap\n- foo:\n  - 1\n  - 2\n- foo:\n  - 3\n  - 4",
    FALSE
  ))
})

test_that("invalid omap causes error", {
  expect_error(yaml.load("--- !omap\nhey!"))
  expect_error(yaml.load("--- !omap\n- sup?"))
})

test_that("expressions are not implicitly converted with warning", {
  expect_warning(
    x <- yaml.load("!expr |\n  function() \n  {\n    'hey!'\n  }"),
    "Evaluating R expressions"
  )
  expect_equal(x, "function() \n{\n  'hey!'\n}")
})

test_that("expressions are explicitly converted without warning", {
  expect_no_warning({
    x <- yaml.load(
      "!expr |\n  function() \n  {\n    'hey!'\n  }",
      eval.expr = TRUE
    )
  })
  expect_equal(class(x), "function")
  expect_equal(x(), "hey!")
})

test_that("expressions are explicitly not converted", {
  x <- yaml.load("!expr 123 + 456", eval.expr = FALSE)
  expect_equal(x, "123 + 456")
})

test_that("invalid expressions cause error", {
  expect_error(yaml.load("!expr |\n  1+", eval.expr = TRUE))
})

test_that("maps are in ordered", {
  x <- yaml.load("{a: 1, b: 2, c: 3}")
  expect_equal(names(x), c("a", "b", "c"))
})

test_that("illegal recursive anchor is handled", {
  expect_warning(x <- yaml.load("&foo {foo: *foo}"), "Unknown anchor: foo")
  expected <- "_yaml.bad-anchor_"
  class(expected) <- "_yaml.bad-anchor_"
  expect_equal(x$foo, expected)
})

test_that("dereferenced aliases have unshared names", {
  x <- yaml.load("{foo: &foo {one: 1, two: 2}, bar: *foo}")
  x$foo$one <- "uno"
  expect_equal(x$bar$one, 1L)
})

test_that("multiple anchors are handled", {
  x <- yaml.load(
    "{foo: &foo {one: 1}, bar: &bar {two: 2}, baz: *foo, quux: *bar}"
  )
  expected <- list(
    foo = list(one = 1),
    bar = list(two = 2),
    baz = list(one = 1),
    quux = list(two = 2)
  )
  expect_equal(x, expected)
})

test_that("quoted strings are preserved", {
  x <- yaml.load("'12345'")
  expect_equal(x, "12345")
})

test_that("inf is loaded properly", {
  result <- yaml.load(".inf\n...\n")
  expect_equal(result, Inf)
})

test_that("negative inf is loaded properly", {
  result <- yaml.load("-.inf\n...\n")
  expect_equal(result, -Inf)
})

test_that("nan is loaded properly", {
  result <- yaml.load(".nan\n...\n")
  expect_equal(result, NaN)
})

test_that("logical na is loaded properly", {
  result <- yaml.load(".na\n...\n")
  expect_equal(result, NA)
})

test_that("numeric na is loaded properly", {
  result <- yaml.load(".na.real\n...\n")
  expect_equal(result, NA_real_)
})

test_that("integer na is loaded properly", {
  result <- yaml.load(".na.integer\n...\n")
  expect_equal(result, NA_integer_)
})

test_that("character na is loaded properly", {
  result <- yaml.load(".na.character\n...\n")
  expect_equal(result, NA_character_)
})

test_that("true is loaded properly from y", {
  result <- yaml.load("y\n...\n")
  expect_equal(result, TRUE)
})

test_that("false is loaded properly from n", {
  result <- yaml.load("n\n...\n")
  expect_equal(result, FALSE)
})

test_that("numeric sequence with missing values loads properly", {
  result <- yaml.load("[1.2, 3.4, .na.real]")
  expect_equal(result, c(1.2, 3.4, NA))
})

test_that("integer sequence with missing values loads properly", {
  result <- yaml.load("[1, 2, .na.integer]")
  expect_equal(result, c(1, 2, NA))
})

test_that("string sequence with missing values loads properly", {
  result <- yaml.load("[foo, bar, .na.character]")
  expect_equal(result, c("foo", "bar", NA))
})

test_that("logical sequence with missing values loads properly", {
  result <- yaml.load("[y, n, .na]")
  expect_equal(result, c(TRUE, FALSE, NA))
})

test_that("numeric sequence with nans loads properly", {
  result <- yaml.load("[1.2, 3.4, .nan]")
  expect_equal(result, c(1.2, 3.4, NaN))
})

test_that("numeric represented in exponential form is loaded properly", {
  expect_equal(yaml.load("1.0e+06"), 1000000)
})

test_that("numeric without leading digits is loaded properly", {
  expect_equal(yaml.load(".9"), 0.9)
})

test_that("integer overflow creates a warning", {
  expect_warning(result <- yaml.load("2147483648"))
  expect_equal(result, NA_integer_)
  expect_warning(result <- yaml.load("2147483649"))
  expect_equal(result, NA_integer_)
})

test_that("numeric overflow creates a warning", {
  expect_warning(result <- yaml.load("1.797693e+309"))
  expect_equal(result, NA_real_)
})

test_that("list of one list is loaded properly", {
  result <- yaml.load("a:\n -\n  - b\n  - c\n")
  expect_equal(result, list(a = list(c("b", "c"))))
})

test_that("override merge precedence", {
  doc <- "[ &one { foo: bar }, { <<: *one, foo: baz } ]"
  expected <- list(list(foo = "bar"), list(foo = "baz"))
  result <- yaml.load(doc, merge.precedence = "override")
  expect_equal(result, expected)
})

test_that("explicit bool tag for true value", {
  doc <- "!!bool 'true'"
  expected <- TRUE
  result <- yaml.load(doc)
  expect_equal(result, expected)
})

test_that("explicit bool tag for false value", {
  doc <- "!!bool 'false'"
  expected <- FALSE
  result <- yaml.load(doc)
  expect_equal(result, expected)
})

test_that("explicit bool tag for na value", {
  doc <- "!!bool '.na'"
  expected <- NA
  result <- yaml.load(doc)
  expect_equal(result, expected)
})

test_that("explicit bool tag for invalid value", {
  doc <- "!!bool foo"
  expected <- NA
  expect_warning(
    result <- yaml.load(doc),
    "NAs introduced by coercion: foo is not a recognized boolean value"
  )
  expect_equal(result, expected)
})

test_that("builtin as handler works", {
  x <- "{a: 1, b: 2, c: 3}"
  expect_no_warning({
    results <- yaml.load(x, handlers = list(int = as.double))
  })
  expect_equal(class(results$a), "numeric")
})

stopifnot(require(RUnit, quietly=TRUE))
stopifnot(require(yaml, quietly=TRUE))

# Set RUnit options
if (!getOption("yaml.verbose", FALSE)) {
  opts <- getOption("RUnit")
  opts$silent <- TRUE
  opts$verbose <- FALSE
  options("RUnit" = opts)
}

# Test helpers
captureWarnings <- function(expr) {
  warnings <- character(0)
  suppressWarnings({
    withCallingHandlers(expr, warning = function(w) {
      warnings <<- c(warnings, w$message)
    })
  })
  warnings
}

checkWarning <- function(expr) {
  warnings <- captureWarnings(expr)
  checkTrue(length(warnings) > 0)
}

checkNamedListEquals <- function(x, y) {
  checkEquals(class(x), class(y))
  checkEquals(length(x), length(y))

  ns <- sort(names(x))
  checkEquals(ns, sort(names(y)))
  for (n in ns) {
    checkEquals(x[[n]], y[[n]])
  }
}

# Define tests
testSuite <- defineTestSuite(name = "yaml tests",
                             dirs = system.file("tests", package = "yaml"),
                             testFileRegexp = "^test_.+",
                             testFuncRegexp = "^test_.+",
                             rngKind = 'Mersenne-Twister',
                             rngNormalKind = 'Inversion')

tests <- runTestSuite(testSuite)

# Print results
printTextProtocol(tests, showDetails = FALSE)

# Return success or failure to R CMD CHECK
if (getErrors(tests)$nFail > 0) {
  stop("TEST FAILED!")
}
if (getErrors(tests)$nErr > 0) {
  stop("TEST HAD ERRORS!")
}
if (getErrors(tests)$nTestFunc < 1) {
  stop("NO TEST FUNCTIONS RUN!")
}

stopifnot(require(RUnit, quietly=TRUE))
stopifnot(require(yaml, quietly=TRUE))

# Set RUnit options
if (!getOption("yaml.verbose", FALSE)) {
  opts <- getOption("RUnit")
  opts$silent <- TRUE
  opts$verbose <- FALSE
  options("RUnit" = opts)
}

# Test helpers
captureWarnings <- function(expr) {
  warnings <- character(0)
  suppressWarnings({
    withCallingHandlers(expr, warning = function(w) {
      warnings <<- c(warnings, w$message)
    })
  })
  warnings
}

checkWarning <- function(expr) {
  warnings <- captureWarnings(expr)
  checkTrue(length(warnings) > 0)
}

checkNamedListEquals <- function(x, y) {
  checkEquals(class(x), class(y))
  checkEquals(length(x), length(y))

  ns <- sort(names(x))
  checkEquals(ns, sort(names(y)))
  for (n in ns) {
    checkEquals(x[[n]], y[[n]])
  }
}

# Define tests
testSuite <- defineTestSuite(name = "yaml tests",
                             dirs = system.file("tests", package = "yaml"),
                             testFileRegexp = "^test_.+",
                             testFuncRegexp = "^test_.+")

tests <- runTestSuite(testSuite)

# Print results
printTextProtocol(tests, showDetails = FALSE)

# Return success or failure to R CMD CHECK
if (getErrors(tests)$nFail > 0) {
  stop("TEST FAILED!")
}
if (getErrors(tests)$nErr > 0) {
  stop("TEST HAD ERRORS!")
}
if (getErrors(tests)$nTestFunc < 1) {
  stop("NO TEST FUNCTIONS RUN!")
}

:: MSVC is preferred.
set CC=cl.exe
set CXX=cl.exe

mkdir build
cd build
cmake ^
    -G "Ninja" ^
    -DCMAKE_INSTALL_PREFIX=%LIBRARY_PREFIX% ^
    -DCMAKE_BUILD_TYPE=Release ^
    -DCMAKE_INSTALL_SYSTEM_RUNTIME_LIBS_SKIP=True ^
    -DBUILD_SHARED_LIBS=ON ^
    %SRC_DIR%
if errorlevel 1 exit 1

:: Build.
cmake --build . --config Release
if errorlevel 1 exit 1

:: Install.
cmake --build . --config Release --target install
if errorlevel 1 exit 1

:: Remove extra output.
del /f "%LIBRARY_PREFIX%\include\config.h"

if not exist "%LIBRARY_INC%\\yaml.h" exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist "%LIBRARY_LIB%\\yaml.lib" exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist "%LIBRARY_BIN%\\yaml.dll" exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0

---
title: "Introduction to yaml"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to yaml}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(yaml)
```

## What is YAML?

YAML is a human-readable data serialization language. With it, you can create
easily readable documents that can be consumed by a variety of programming
languages.

For example, here's a map of baseball teams per league:

```yaml
american:
- Boston Red Sox
- Detroit Tigers
- New York Yankees
national:
- New York Mets
- Chicago Cubs
- Atlanta Braves
```

And a data dictionary specification:

```yaml
- field: ID
  description: primary identifier
  type: integer
  primary key: yes
- field: DOB
  description: date of birth
  type: date
  format: yyyy-mm-dd
- field: State
  description: state of residence
  type: string
```

## Parsing YAML

`yaml.load()` parses a YAML document from a string:

```{r}
yaml.load("
- 1
- 2
- 3
")
```

`yaml.load_file()` and `read_yaml()` read YAML from a file or connection.

### Scalars

A YAML scalar is the basic building block of YAML documents. The parser
automatically determines the type:

```{r}
yaml.load("1.2345")
yaml.load("true")
yaml.load("hello")
```

### Sequences

A YAML sequence is a list of elements. If all elements are uniform,
`yaml.load()` returns a vector of that type. Otherwise, it returns a list:

```{r}
# Uniform sequence -> vector
yaml.load("
- 1
- 2
- 3
")

# Mixed sequence -> list
yaml.load("
- 1
- hello
- true
")
```

### Maps

A YAML map is a list of paired keys and values. By default, `yaml.load()`
returns a named list:

```{r}
yaml.load("
one: 1
two: 2
three: 3
")
```

Since YAML map keys can be almost anything (not just strings), you can preserve
the data type of keys with `as.named.list = FALSE`. This creates a `keys`
attribute instead of coercing keys to strings:

```{r}
yaml.load("
1: one
2: two
", as.named.list = FALSE)
```

### Custom handlers

You can customize parsing with handler functions. Handlers are passed to
`yaml.load()` as a named list, where each name is the YAML type to handle:

```{r}
# Add 100 to all integers
yaml.load("123", handlers = list(int = function(x) as.integer(x) + 100))
```

#### Sequence handlers

Sequence handlers receive a list and can transform it:

```{r}
yaml.load("
- 1
- 2
- 3
", handlers = list(seq = function(x) sum(as.numeric(x))))
```

#### Map handlers

Map handlers receive a named list (or a list with a `keys` attribute):

```{r}
yaml.load("
a:
- 1
- 2
b:
- 3
- 4
", handlers = list(map = function(x) as.data.frame(x)))
```

## Emitting YAML

`as.yaml()` converts R objects to YAML strings:

```{r}
cat(as.yaml(1:5))
```

`write_yaml()` writes the result directly to a file or connection.

### Formatting options

#### indent

Control indentation depth (default is 2):

```{r}
cat(as.yaml(list(foo = list(bar = "baz")), indent = 4))
```

#### indent.mapping.sequence

By default, sequences within a mapping are not indented:

```{r}
cat(as.yaml(list(foo = 1:3)))
```

Set `indent.mapping.sequence = TRUE` to indent them:

```{r}
cat(as.yaml(list(foo = 1:3), indent.mapping.sequence = TRUE))
```

#### column.major

Controls how data frames are converted. When `TRUE` (default), data frames
are emitted column-wise:

```{r}
x <- data.frame(a = 1:2, b = 3:4)
cat(as.yaml(x, column.major = TRUE))
```

When `FALSE`, data frames are emitted row-wise:

```{r}
cat(as.yaml(x, column.major = FALSE))
```

### Custom handlers

Specify custom handler functions for R object classes:

```{r}
cat(as.yaml(
  Sys.Date(),
  handlers = list(Date = function(x) format(x, "%Y/%m/%d"))
))
```

### YAML 1.2 logical handling

For YAML 1.2-like logical output, use `verbatim_logical`:

```{r}
cat(as.yaml(c(TRUE, FALSE), handlers = list(logical = verbatim_logical)))
```

### Verbatim text

Character vectors with class `"verbatim"` are not quoted except when required
by the YAML specification:

```{r}
result <- c("true", "false")
class(result) <- "verbatim"
cat(as.yaml(result))
```

### Quoted strings

Force quoting with the `"quoted"` attribute:

```{r}
port <- "80:80"
attr(port, "quoted") <- TRUE
cat(as.yaml(list(ports = list(port))))
```

### Custom tags

Set YAML tags with the `"tag"` attribute:

```{r}
x <- 1:3
attr(x, "tag") <- "!custom"
cat(as.yaml(x))
```

=head1 NAME

YAML - YAML Ain't Markup Language (tm)

=head1 SYNOPSIS

    use YAML;
    
    # Load a YAML stream of 3 YAML documents into Perl data structures.
    my ($hashref, $arrayref, $string) = Load(<<'...');
    ---
    name: ingy
    age: old
    weight: heavy
    # I should comment that I also like pink, but don't tell anybody.
    favorite colors:
        - red
        - white
        - blue
    ---
    - Clark Evans
    - Oren Ben-Kiki
    - Brian Ingerson
    --- >
    You probably think YAML stands for "Yet Another Markup Language". It
    ain't! YAML is really a data serialization language. But if you want
    to think of it as a markup, that's OK with me. A lot of people try
    to use XML as a serialization format.
    
    "YAML" is catchy and fun to say. Try it. "YAML, YAML, YAML!!!"
    ...
    
    # Dump the Perl data structures back into YAML.
    print Dump($string, $arrayref, $hashref); 
    
    # YAML::Dump is used the same way you'd use Data::Dumper::Dumper
    use Data::Dumper;
    print Dumper($string, $arrayref, $hashref); 

=head1 DESCRIPTION

The YAML.pm module implements a YAML Loader and Dumper based on the YAML
1.0 specification. L<http://www.yaml.org/spec/>

YAML is a generic data serialization language that is optimized for
human readability. It can be used to express the data structures of most
modern programming languages. (Including Perl!!!)

For information on the YAML syntax, please refer to the YAML
specification.

=head1 WHY YAML IS COOL

=over 4

=item YAML is readable for people. 

It makes clear sense out of complex data structures. You should find
that YAML is an exceptional data dumping tool. Structure is shown
through indentation, YAML supports recursive data, and hash keys are
sorted by default. In addition, YAML supports several styles of scalar
formatting for different types of data.

=item YAML is editable.

YAML was designed from the ground up to be an excellent syntax for
configuration files. Almost all programs need configuration files, so
why invent a new syntax for each one? And why subject users to the
complexities of XML or native Perl code?

=item YAML is multilingual.

Yes, YAML supports Unicode. But I'm actually referring to programming
languages. YAML was designed to meet the serialization needs of Perl,
Python, Ruby, Tcl, PHP and Java. It was also designed to be
interoperable between those languages. That means any YAML serialization
produced by Perl can be processed by Python, and be guaranteed to return
the data structure intact. (Even if it contained Perl specific
structures like GLOBs)

=item YAML is taint safe.

Using modules like Data::Dumper for serialization is fine as long as you
can be sure that nobody can tamper with your data files or
transmissions. That's because you need to use Perl's C<eval()> built-in
to deserialize the data. Somebody could add a snippet of Perl to erase
your files.

YAML's parser does not need to eval anything.

=item YAML is full featured.

YAML can accurately serialize all of the common Perl data structures and
deserialize them again without losing data relationships. Although it is
not 100% perfect (no serializer is or can be perfect), it fares as well
as the popular current modules: Data::Dumper, Storable, XML::Dumper and
Data::Denter.

YAML.pm also has the ability to handle code (subroutine) references and
typeglobs. (Still experimental) These features are not found in Perl's
other serialization modules.

=item YAML is extensible.

The YAML language has been designed to be flexible enough to solve it's
own problems. The markup itself has 3 basic construct which resemble
Perl's hash, array and scalar. By default, these map to their Perl
equivalents. But each YAML node also supports a type (or "transfer
method") which can cause that node to be interpreted in a completely
different manner. That's how YAML can support oddball structures like
Perl's typeglob.

=item YAML.pm plays well with others.

YAML has been designed to interact well with other Perl Modules like POE
and Time::Object. (date support coming soon)

=back

=head1 USAGE

=head2 Exported Functions

The following functions are exported by YAML.pm by default when you use
YAML.pm like this:

    use YAML;

To prevent YAML.pm from exporting functions, say:

    use YAML ();

=over 4

=item Dump(list-of-Perl-data-structures)

Turn Perl data into YAML. This function works very much like
Data::Dumper::Dumper(). It takes a list of Perl data strucures and
dumps them into a serialized form. It returns a string containing the
YAML stream. The structures can be references or plain scalars.

=item Load(string-containing-a-YAML-stream)

Turn YAML into Perl data. This is the opposite of Dump. Just like
Storable's thaw() function or the eval() function in relation to
Data::Dumper. It parses a string containing a valid YAML stream into a
list of Perl data structures. 

=item Store()

This function is deprecated, and now refered to as Dump. It is still available
for the time being, but will generate a warning if you are using -w. You
B<are> using -w, aren't you? :)

The reason for this deprecation is that the YAML spec talks about programs
called Loaders and Dumpers. "Storers" is too hard to say, I guess...

=back

=head2 Exportable Functions

=over 4

=item DumpFile(filepath, list)

Writes the YAML stream to a file instead of just returning a string.

=item LoadFile(filepath)

Reads the YAML stream from a file instead of a string.

=item Bless(perl-node, [yaml-node | class-name])

Associate a normal Perl node, with a yaml node. A yaml node is an object
tied to the YAML::Node class. The second argument is either a yaml node
that you've already created or a class (package) name that supports a
yaml_dump() function. A yaml_dump() function should take a perl node and
return a yaml node. If no second argument is provided, Bless will create a
yaml node. This node is not returned, but can be retrieved with the Blessed()
function.

Here's an example of how to use Bless. Say you have a hash containing three
keys, but you only want to dump two of them. Furthermore the keys must be
dumped in a certain order. Here's how you do that:

    use YAML qw(Dump Bless);
    $hash = {apple => 'good', banana => 'bad', cauliflower => 'ugly'};
    print Dump $hash;
    Bless($hash)->keys(['banana', 'apple']);
    print Dump $hash;

produces:

    --- #YAML:1.0
    apple: good
    banana: bad
    cauliflower: ugly
    --- #YAML:1.0
    banana: bad
    apple: good

Bless returns the tied part of a yaml-node, so that you can call the
YAML::Node methods. This is the same thing that YAML::Node::ynode()
returns. So another way to do the above example is:

    use YAML qw(:all);
    use YAML::Node;
    $hash = {apple => 'good', banana => 'bad', cauliflower => 'ugly'};
    print Dump $hash;
    Bless($hash);
    $ynode = ynode(Blessed($hash));
    $ynode->keys(['banana', 'apple']);
    print Dump $hash;

=item Blessed(perl-node)

Returns the yaml node that a particular perl node is associated with
(see above). Returns undef if the node is not (YAML) blessed.

=item Dumper()

Alias to Dump(). For Data::Dumper fans.

=item freeze() and thaw()

Aliases to Dump() and Load(). For Storable fans.

This will also allow YAML.pm to be plugged directly into modules like POE.pm,
that use the freeze/thaw API for internal serialization.

=back

=head2 Exportable Function Groups

This is a list of the various groups of exported functions that you can import
using the following syntax:

    use YAML ':groupname';

=over 4

=item all

Imports Dump(), Load(), Store(), DumpFile(), LoadFile(), Bless() and Blessed().

=item POE

Imports freeze() and thaw().

=item Storable

Imports freeze() and thaw().

=back

=head2 Class Methods

YAML can also be used in an object oriented manner. At this point it
offers no real advantage. This interface will be improved in a later
release.

=over 4

=item new()

New returns a new YAML object. For example:

    my $y = YAML->new;
    $y->Indent(4);
    $y->dump($foo, $bar);

=back

=head2 Object Methods

=over 4

=item dump()

OO version of Dump().

=item load()

OO version of Load().

=back

=head2 Options

YAML options are set using a group of global variables in the YAML
namespace. This is similar to how Data::Dumper works.

For example, to change the indentation width, do something like:

    local $YAML::Indent = 3;

The current options are:

=over 4

=item Indent

This is the number of space characters to use for each indentation level
when doing a Dump(). The default is 2.

By the way, YAML can use any number of characters for indentation at any
level. So if you are editing YAML by hand feel free to do it anyway that
looks pleasing to you; just be consistent for a given level.

=item UseHeader

Default is 1. (true)

This tells YAML.pm whether to use a separator string for a Dump
operation. This only applies to the first document in a stream.
Subsequent documents must have a YAML header by definition.

=item UseVersion

Default is 1. (true)

Tells YAML.pm whether to include the YAML version on the
separator/header.

The canonical form is:

    --- YAML:1.0

=item SortKeys

Default is 1. (true)

Tells YAML.pm whether or not to sort hash keys when storing a document. 

YAML::Node objects can have their own sort order, which is usually what
you want. To override the YAML::Node order and sort the keys anyway, set
SortKeys to 2.

=item AnchorPrefix

Default is ''.

Anchor names are normally numeric. YAML.pm simply starts with '1' and
increases by one for each new anchor. This option allows you to specify a
string to be prepended to each anchor number.

=item UseCode

Setting the UseCode option is a shortcut to set both the DumpCode and
LoadCode options at once. Setting UseCode to '1' tells YAML.pm to dump
Perl code references as Perl (using B::Deparse) and to load them back
into memory using eval(). The reason this has to be an option is that
using eval() to parse untrusted code is, well, untrustworthy. Safe
deserialization is one of the core goals of YAML.

=item DumpCode

Determines if and how YAML.pm should serialize Perl code references. By
default YAML.pm will dump code references as dummy placeholders (much
like Data::Dumper). If DumpCode is set to '1' or 'deparse', code
references will be dumped as actual Perl code.

DumpCode can also be set to a subroutine reference so that you can
write your own serializing routine. YAML.pm passes you the code ref. You
pass back the serialization (as a string) and a format indicator. The
format indicator is a simple string like: 'deparse' or 'bytecode'.

=item LoadCode

LoadCode is the opposite of DumpCode. It tells YAML if and how to
deserialize code references. When set to '1' or 'deparse' it will use
C<eval()>. Since this is potentially risky, only use this option if you
know where your YAML has been.

LoadCode can also be set to a subroutine reference so that you can write
your own deserializing routine. YAML.pm passes the serialization (as a
string) and a format indicator. You pass back the code reference.

=item UseBlock

YAML.pm uses heuristics to guess which scalar style is best for a given
node. Sometimes you'll want all multiline scalars to use the 'block'
style. If so, set this option to 1.

NOTE: YAML's block style is akin to Perl's here-document. 

=item ForceBlock

Force every possible scalar to be block formatted. NOTE: Escape characters
cannot be formatted in a block scalar.

=item UseFold

If you want to force YAML to use the 'folded' style for all multiline
scalars, then set $UseFold to 1.

NOTE: YAML's folded style is akin to the way HTML folds text,
      except smarter.

=item UseAliases

YAML has an alias mechanism such that any given structure in memory gets
serialized once. Any other references to that structure are serialized
only as alias markers. This is how YAML can serialize duplicate and
recursive structures.

Sometimes, when you KNOW that your data is nonrecursive in nature, you
may want to serialize such that every node is expressed in full. (ie as
a copy of the original). Setting $YAML::UseAliases to 0 will allow you
to do this. This also may result in faster processing because the lookup
overhead is by bypassed.

THIS OPTION CAN BE DANGEROUS. *If* your data is recursive, this option
*will* cause Dump() to run in an endless loop, chewing up your computers
memory. You have been warned.

=item CompressSeries

Default is 1.

Compresses the formatting of arrays of hashes:

    -
      foo: bar
    - 
      bar: foo

becomes:

    - foo: bar
    - bar: foo

Since this output is usually more desirable, this option is turned on by
default.

=back

=head1 YAML TERMINOLOGY

YAML is a full featured data serialization language, and thus has its
own terminology.

It is important to remember that although YAML is heavily influenced by
Perl and Python, it is a language in it's own right, not merely just a
representation of Perl structures.

YAML has three constructs that are conspicuously similar to Perl's hash,
array, and scalar. They are called mapping, sequence, and string
respectively. By default, they do what you would expect. But each
instance may have an explicit or implicit type that makes it behave
differently. In this manner, YAML can be extended to represent Perl's
Glob or Python's tuple, or Ruby's Bigint.

=over 4

=item stream

A YAML stream is the full sequence of bytes that a YAML parser would
read or a YAML emitter would write. A stream may contain one or more YAML
documents separated by YAML headers.

    ---
    a: mapping
    foo: bar
    ---
    - a
    - sequence

=item document

A YAML document is an independent data structure representation within a
stream. It is a top level node.

    --- YAML:1.0
    This: top level mapping
    is:
        - a
        - YAML
        - document

=item node

A YAML node is the representation of a particular data stucture. Nodes
may contain other nodes. (In Perl terms, nodes are like scalars.
Strings, arrayrefs and hashrefs. But this refers to the serialized
format, not the in-memory structure.)

=item transfer method

This is similar to a type. It indicates how a particular YAML node
serialization should be transferred into or out of memory. For instance
a Foo::Bar object would use the transfer 'perl/Foo::Bar':

    - !perl/Foo::Bar
        foo: 42
        bar: stool

=item collection

A collection is the generic term for a YAML data grouping. YAML has two
types of collections: mappings and sequences. (Similar to hashes and arrays)

=item mapping

A mapping is a YAML collection defined by key/value pairs. By default YAML
mappings are loaded into Perl hashes.

    a mapping:
        foo: bar
        two: times two is 4

=item sequence

A sequence is a YAML collection defined by an ordered list of elements. By
default YAML sequences are loaded into Perl arrays.

    a sequence:
        - one bourbon
        - one scotch
        - one beer

=item scalar

A scalar is a YAML node that is a single value. By default YAML scalars
are loaded into Perl scalars.

    a scalar key: a scalar value

YAML has many styles for representing scalars. This is important because
varying data will have varying formatting requirements to retain the
optimum human readability.

=item simple scalar

This is a single line of unquoted text. All simple scalars are automatic
candidates for "implicit transferring". This means that their B<type> is
determined automatically by examination. Unless they match a set of
predetermined YAML regex patterns, they will raise a parser exception.
The typical uses for this are simple alpha strings, integers, real
numbers, dates, times and currency.

    - a simple string
    - -42
    - 3.1415
    - 12:34
    - 123 this is an error

=item single quoted scalar

This is similar to Perl's use of single quotes. It means no escaping and
no implicit transfer. It must be used on a single line.

    - 'When I say ''\n'' I mean "backslash en"'

=item double quoted scalar

This is similar to Perl's use of double quotes. Character escaping can
be used. There is no implicit transfer and it must still be single line.

    - "This scalar\nhas two lines, and a bell -->\a"

=item folded scalar

This is a multiline scalar which begins on the next line. It is
indicated by a single closing brace. It is unescaped like the single
quoted scalar. Line folding is also performed.

    - > 
     This is a multiline scalar which begins on
     the next line. It is indicated by a single
     carat. It is unescaped like the single
     quoted scalar. Line folding is also
     performed.

=item block scalar

This final multiline form is akin to Perl's here-document except that
(as in all YAML data) scope is indicated by indentation. Therefore, no
ending marker is required. The data is verbatim. No line folding.

    - |
        QTY  DESC          PRICE  TOTAL
        ---  ----          -----  -----
          1  Foo Fighters  $19.95 $19.95
          2  Bar Belles    $29.95 $59.90

=item parser

A YAML processor has four stages: parse, load, dump, emit. 

A parser parses a YAML stream. YAML.pm's Load() function contains a
parser.

=item loader

The other half of the Load() function is a loader. This takes the
information from the parser and loads it into a Perl data structure.

=item dumper

The Dump() function consists of a dumper and an emitter. The dumper
walks through each Perl data structure and gives info to the emitter.

=item emitter

The emitter takes info from the dumper and turns it into a YAML stream. 

NOTE: 
In YAML.pm the parser/loader and the dumper/emitter code are currently
very closely tied together. When libyaml is written (in C) there will be
a definite separation. libyaml will contain a parser and emitter, and
YAML.pm (and YAML.py etc) will supply the loader and dumper.

=back

For more information please refer to the immensely helpful YAML
specification available at L<http://www.yaml.org/spec/>.

=head1 ysh - The YAML Shell

The YAML distribution ships with a script called 'ysh', the YAML shell.
ysh provides a simple, interactive way to play with YAML. If you type in
Perl code, it displays the result in YAML. If you type in YAML it turns
it into Perl code.

To run ysh, (assuming you installed it along with YAML.pm) simply type:

    ysh [options]

Please read L<ysh> for the full details. There are lots of options.

=head1 BUGS & DEFICIENCIES

If you find a bug in YAML, please try to recreate it in the YAML Shell
with logging turned on ('ysh -L'). When you have successfully reproduced
the bug, please mail the LOG file to the author (ingy@cpan.org).

WARNING: This is *ALPHA* code. 

BIGGER WARNING: This is *TRIAL1* of the YAML 1.0 specification. The YAML
syntax may change before it is finalized. Based on past experience, it
probably will change. The authors of this spec have worked for over a
year putting together YAML 1.0, and we have flipped it on it's
syntactical head almost every week. We're a fickle lot, we are. So use
this at your own risk!!!

=over 4

=item Circular Leaves

YAML is quite capable of serializing circular references. And for the
most part it can deserialize them correctly too. One notable exception
is a reference to a leaf node containing itself. This is hard to do from
pure Perl in any elegant way. The "canonical" example is:

    $foo = \$foo;

This serializes fine, but I can't parse it correctly yet. Unfortunately,
every wiseguy programmer in the world seems to try this first when you
ask them to test your serialization module. Even though it is of almost
no real world value. So please don't report this bug unless you have a
pure Perl patch to fix it for me.

By the way, similar non-leaf structures Dump and Load just fine:

    $foo->[0] = $foo;

You can test these examples using 'ysh -r'. This option makes sure that
the example can be deserialized after it is serialized. We call that
"roundtripping", thus the '-r'.

=item Unicode

Unicode is not yet supported. The YAML specification dictates that all
strings be unicode, but this early implementation just uses ASCII.

=item Structured Keys

Python, Java and perhaps others support using any data type as the
key to a hash. YAML also supports this. Perl5 only uses strings as
hash keys.

YAML.pm can currently parse structured keys, but their meaning gets lost
when they are loaded into a Perl hash. Consider this example using the
YAML Shell:

    ysh > ---
    yaml> ?
    yaml>  foo: bar
    yaml> : baz
    yaml> ...
    $VAR1 = {
              'HASH(0x1f1d20)' => 'baz'
            };
    ysh >

YAML.pm will need to be fixed to preserve these keys somehow. Why?
Because if YAML.pm gets a YAML document from YAML.py it must be able to
return it with the Python data intact.

=item Globs, Subroutines, Regexes and File Handles

As far as I know, other Perl serialization modules are not capable of
serializing and deserializing typeglobs, subroutines (code refs),
regexes and file handles. YAML.pm has dumping capabilities for all of these.
Loading them may produce wild results. Take care.

NOTE: For a (huge) dump of Perl's global guts, try:

    perl -MYAML -e '$YAML::UseCode=1; print Dump \%main::'

To limit this to a single namespace try:

    perl -MCGI -MYAML -e '$YAML::UseCode=1; print Dump \%CGI::'

=item Speed

This is a pure Perl implementation that has been optimized for
programmer readability, not for computational speed. 

Neil Watkiss and Clark Evans are currently developing libyaml, the
official C implementation of the YAML parser and emitter. YAML.pm will
be refactoring to use this library once it is stable. Other languages
like Python, Tcl, PHP, Ruby, JavaScript and Java can make use of the
same core library.

Please join us on the YAML mailing list if you are interested in
implementing something. 

L<https://lists.sourceforge.net/lists/listinfo/yaml-core>

=item Streaming Access

This module Dumps and Loads in one operation. There is no interface
for parsing or emitting a YAML stream one node at a time. It's all
or nothing.

An upcoming release will have support for incremental parsing.
Incremental dumping is harder. Stay tuned.

=back

=head1 RESOURCES

Please read L<YAML::Node> for advanced YAML features.

L<http://www.yaml.org> is the official YAML website.

L<http://www.yaml.org/spec/> is the YAML 1.0 specification.

L<http://wiki.yaml.org/spec/> is the official YAML wiki.

YAML has been registered as a Source Forge project.
(L<http://www.sourceforge.net>) Currently we are only using the mailing
list facilities there.

=head1 IMPLEMENTATIONS

This is the first implementation of YAML functionality based on the 1.0
specification.

The following people have shown an interest in doing implementations.
Please contact them if you are also interested in writing an
implementation.

    ---
    - name:    Neil Watkiss
      project: 
        - libyaml
        - YAML mode for the vim editor
      email:   nwatkiss@ttul.org

    - name:    Brian Ingerson
      project: YAML.pm, libyaml Perl binding
      email:   ingy@ttul.org
    
    - name:    Clark Evans
      project: libyaml, Python binding
      email:   cce@clarkevans.com

    - name:    Oren Ben-Kiki
      project: Java Loader/Dumper
      email:   orenbk@richfx.com

    - name:    Paul Prescod
      project: YAML Antagonist/Anarchist
      email:   paul@prescod.net

    - name:    Ryan King
      project: YAML test specialist
      email:   rking@panoptic.com

    - name:    Steve Howell
      project: Python and Ruby implementations
      email:   showell@zipcon.net

    - name:    Patrick Leboutillier
      project: Java Loader/Dumper
      email:   patrick_leboutillier@hotmail.com

    - name:    Shane Caraveo
      project: PHP Loader/Dumper
      email:   shanec@activestate.com

    - name:    Brian Quinlan
      project: Python Loader/Dumper
      email:   brian@sweetapp.com

    - name:    Jeff Hobbs
      project: Tcl Loader/Dumper
      email:   jeff@hobbs.org

    - name:    Claes Jacobsson
      project: JavaScript Loader/Dumper
      email:   claes@contiller.se

=head1 AUTHOR

Brian Ingerson <INGY@cpan.org> is resonsible for YAML.pm.

The YAML language is the result of a ton of collaboration between Oren
Ben-Kiki, Clark Evans and Brian Ingerson. Several others have added help
along the way.

Neil Watkiss is pioneering libyaml. Bless that boy!

Ryan King offered much help on the 0.35 release. The XP advocate
extraordinaire, help me refactor my entire test suite into its
current form. Regression tests are extremely important to the success
of this project.

=head1 COPYRIGHT

Copyright (c) 2001, 2002. Brian Ingerson. All rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbatim_logical.R
\name{verbatim_logical}
\alias{verbatim_logical}
\title{Alternative logical handler}
\usage{
verbatim_logical(x)
}
\arguments{
\item{x}{Logical vector to convert to \code{true}/\code{false}.}
}
\value{
Returns a vector of strings of either \code{true} or \code{false} of
class \code{verbatim}.
}
\description{
A yaml handler function that causes logical vectors to emit
\code{true}/\code{false} instead of \code{yes}/\code{no} values.
}
\details{
Pass this function to \code{\link[=as.yaml]{as.yaml()}} as part of the
\code{handlers} argument list like \code{list(logical = verbatim_logical)}.
}
\examples{

vector <- c(TRUE, FALSE, TRUE)

as.yaml(vector, handlers=list(logical=verbatim_logical))

}
\seealso{
\code{\link[=as.yaml]{as.yaml()}}
}
\author{
Charles Dupont and James Goldie (jimjam-slam)
}
\keyword{data}
\keyword{manip}

=encoding utf8

=head1 NAME

YAML::Mo - Mo Base Class for YAML

=head1 AUTHOR

Ingy döt Net <ingy@ingy.net>

=head1 COPYRIGHT AND LICENSE

Copyright (c) 2011. Ingy döt Net.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

See http://www.perl.com/perl/misc/Artistic.html

=cut

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yaml.load.R, R/yaml.load_file.R
\name{yaml.load}
\alias{yaml.load}
\alias{yaml.load_file}
\title{Convert a YAML string into R objects}
\usage{
yaml.load(
  string,
  as.named.list = TRUE,
  handlers = NULL,
  error.label = NULL,
  eval.expr = getOption("yaml.eval.expr", FALSE),
  merge.precedence = c("order", "override"),
  merge.warning = FALSE
)

yaml.load_file(input, error.label, readLines.warn = TRUE, ...)
}
\arguments{
\item{string}{The YAML string to be parsed.}

\item{as.named.list}{Whether or not to return a named list for maps (TRUE by
default).}

\item{handlers}{Named list of custom handler functions for YAML types (see
Details).}

\item{error.label}{A label to prepend to error messages (see Details).}

\item{eval.expr}{Whether or not to evaluate expressions found in the YAML
document (see Details).}

\item{merge.precedence}{Precedence behavior during map merges (see
Details).}

\item{merge.warning}{Whether or not to warn about ignored key/value pairs
during map merges.}

\item{input}{A filename or connection; if \code{input} is a filename, that
file must be encoded in UTF-8.}

\item{readLines.warn}{Logical (default: TRUE). Suppress warnings from
readLines used inside read_yaml.}

\item{...}{Arguments to pass to yaml.load.}
}
\value{
If the root YAML object is a map, a named list or list with an
attribute of 'keys' is returned.  If the root object is a sequence, a list
or vector is returned, depending on the contents of the sequence.  A vector
of length 1 is returned for single objects.
}
\description{
Parse a YAML string and return R objects.
}
\details{
Use \code{yaml.load} to load a YAML string.  For files and connections, use
\code{yaml.load_file}, which calls \code{yaml.load} with the contents of the
specified file or connection.

Sequences of uniform data (e.g. a sequence of integers) are converted into
vectors.  If the sequence is not uniform, it's returned as a list. Maps are
converted into named lists by default, and all the keys in the map are
converted to strings.  If you don't want the keys to be coerced into
strings, set \code{as.named.list} to FALSE.  When it's FALSE, a list will be
returned with an additional attribute named 'keys', which is a list of the
un-coerced keys in the map (in the same order as the main list).

You can specify custom handler functions via the \code{handlers} argument.
This argument must be a named list of functions, where the names are the
YAML types (i.e., 'int', 'float', 'seq', etc).  The functions you provide
will be passed one argument.  Custom handler functions for string types (all
types except sequence and map) will receive a character vector of length 1.
Custom sequence functions will be passed a list of objects.  Custom map
functions will be passed the object that the internal map handler creates,
which is either a named list or a list with a 'keys' attribute (depending on
\code{as.named.list}).  ALL functions you provide must return an object.
See the examples for custom handler use.

You can specify a label to be prepended to error messages via the
\code{error.label} argument.  When using \code{yaml.load_file}, you can
either set the \code{error.label} argument explicitly or leave it missing.
If missing, \code{yaml.load_file} will make an educated guess for the value
of \code{error.label} by either using the specified filename (when
\code{input} is a character vector) or using the description of the supplied
connection object (via the \code{summary} function).  You can explicitly set
\code{error.label} to \code{NULL} if you don't want to use this
functionality.

There is a built-in handler that will evaluate expressions that are tagged
with the \sQuote{!expr} tag.  Currently this handler is disabled by default
for security reasons.  If a \sQuote{!expr} tag exists and this is set to
FALSE a warning will occur. Alternately, you can set the option named
\sQuote{yaml.eval.expr} via the \code{options} function to turn on
evaluation.

The \code{merge.precedence} parameter controls how merge keys are handled.
The YAML merge key specification is not specific about how key/value
conflicts are resolved during map merges.  As a result, various YAML library
implementations vary in merge key behavior (notably Python and Ruby).  This
package's default behavior (when \code{merge.precedence} is \sQuote{order})
is to give precedence to key/value pairs that appear first.  If you set
\code{merge.precedence} to \sQuote{override}, natural map key/value pairs
will override any duplicate keys found in merged maps, regardless of order.
This is the default behavior in Python's YAML library.

This function uses the YAML parser provided by libyaml, which conforms to
the YAML 1.1 specification.
}
\examples{

  yaml.load("- hey\n- hi\n- hello")
  yaml.load("foo: 123\nbar: 456")
  yaml.load("- foo\n- bar\n- 3.14")
  yaml.load("foo: bar\n123: 456", as.named.list = FALSE)

\dontrun{
  # reading from a file (uses readLines internally)
  filename <- tempfile()
  cat("foo: 123", file=filename, sep="\n")
  yaml.load_file(filename)
}

  # custom scalar handler
  my.float.handler <- function(x) { as.numeric(x) + 123 }
  yaml.load("123.456", handlers=list("float#fix"=my.float.handler))

  # custom sequence handler
  yaml.load("- 1\n- 2\n- 3", handlers=list(seq=function(x) { as.integer(x) + 3 }))

  # custom map handler
  yaml.load("foo: 123", handlers=list(map=function(x) { x$foo <- x$foo + 123; x }))

  # handling custom types
  yaml.load("!sqrt 555", handlers=list(sqrt=function(x) { sqrt(as.integer(x)) }))
  yaml.load("!foo\n- 1\n- 2", handlers=list(foo=function(x) { as.integer(x) + 1 }))
  yaml.load("!bar\none: 1\ntwo: 2", handlers=list(bar=function(x) { x$one <- "one"; x }))

  # loading R expressions
  # NOTE: this will not be done by default in the near future
  doc <- yaml.load("inc: !expr function(x) x + 1", eval.expr=TRUE)
  doc$inc(1)

  # adding a label to error messages
  try(yaml.load("*", error.label = "foo"))

}
\references{
YAML: http://yaml.org

libyaml: https://pyyaml.org/wiki/LibYAML

YAML merge specification: http://yaml.org/type/merge.html
}
\seealso{
\code{\link[=as.yaml]{as.yaml()}}
}
\author{
Jeremy Stephens \href{mailto:jeremy.f.stephens@vumc.org}{jeremy.f.stephens@vumc.org}
}
\keyword{data}
\keyword{manip}
\keyword{programming}

=pod

=for comment
DO NOT EDIT. This Pod was generated by Swim v0.1.43.
See http://github.com/ingydotnet/swim-pm#readme

=encoding utf8

=head1 NAME

YAML::Marshall - YAML marshalling class you can mixin to your classes

=head1 SYNOPSIS

    package Bar;
    use Foo -base;
    use YAML::Marshall -mixin;

=head1 DESCRIPTION

For classes that want to handle their own YAML serialization.

=head1 AUTHOR

ingy döt Net <ingy@cpan.org>

=head1 COPYRIGHT

Copyright 2001-2014. Ingy döt Net

This program is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_yaml.R
\name{read_yaml}
\alias{read_yaml}
\title{Read a YAML file}
\usage{
read_yaml(
  file,
  fileEncoding = "UTF-8",
  text,
  error.label,
  readLines.warn = TRUE,
  ...
)
}
\arguments{
\item{file}{Either a character string naming a file or a \link{connection}
open for reading.}

\item{fileEncoding}{Character string: if non-empty declares the encoding
used on a file (not a connection) so the character data can be re-encoded.
See \code{\link[=file]{file()}}.}

\item{text}{Character string: if \code{file} is not supplied and this is,
then data are read from the value of \code{text} via a text connection.
Notice that a literal string can be used to include (small) data sets
within R code.}

\item{error.label}{A label to prepend to error messages (see Details).}

\item{readLines.warn}{Logical (default: TRUE). Suppress warnings from
readLines used inside read_yaml.}

\item{...}{Arguments to pass to \code{\link[=yaml.load]{yaml.load()}}.}
}
\value{
If the root YAML object is a map, a named list or list with an
attribute of 'keys' is returned.  If the root object is a sequence, a list
or vector is returned, depending on the contents of the sequence.  A vector
of length 1 is returned for single objects.
}
\description{
Read a YAML document from a file and create an R object from it
}
\details{
This function is a convenient wrapper for \code{\link[=yaml.load]{yaml.load()}} and is a
nicer alternative to \code{\link[=yaml.load_file]{yaml.load_file()}}.

You can specify a label to be prepended to error messages via the
\code{error.label} argument.  If \code{error.label} is missing,
\code{read_yaml} will make an educated guess for the value of
\code{error.label} by either using the specified filename (when \code{file}
is a character vector) or using the description of the supplied connection
object (via the \code{summary} function).  If \code{text} is used, the
default value of \code{error.label} will be \code{NULL}.
}
\examples{

\dontrun{
  # reading from a file connection
  filename <- tempfile()
  cat("test: data\n", file = filename)
  con <- file(filename, "r")
  read_yaml(con)
  close(con)

  # using a filename to specify input file
  read_yaml(filename)
}

  # reading from a character vector
  read_yaml(text="- hey\n- hi\n- hello")

}
\references{
YAML: http://yaml.org

libyaml: https://pyyaml.org/wiki/LibYAML
}
\seealso{
\code{\link[=yaml.load]{yaml.load()}}, \code{\link[=write_yaml]{write_yaml()}},
\code{\link[=yaml.load_file]{yaml.load_file()}}
}
\author{
Jeremy Stephens \href{mailto:jeremy.f.stephens@vumc.org}{jeremy.f.stephens@vumc.org}
}
\keyword{data}
\keyword{manip}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.yaml.R
\name{as.yaml}
\alias{as.yaml}
\title{Convert an R object into a YAML string}
\usage{
as.yaml(
  x,
  line.sep = c("\\n", "\\r\\n", "\\r"),
  indent = 2,
  omap = FALSE,
  column.major = TRUE,
  unicode = TRUE,
  precision = getOption("digits"),
  indent.mapping.sequence = FALSE,
  handlers = NULL
)
}
\arguments{
\item{x}{The object to be converted.}

\item{line.sep}{The line separator character(s) to use.}

\item{indent}{The number of spaces to use for indenting.}

\item{omap}{Determines whether or not to convert a list to a YAML omap; see
Details.}

\item{column.major}{Determines how to convert a data.frame; see Details.}

\item{unicode}{Determines whether or not to allow unescaped unicode
characters in output.}

\item{precision}{Number of significant digits to use when formatting numeric
values.}

\item{indent.mapping.sequence}{Determines whether or not to indent sequences
in mapping context.}

\item{handlers}{Named list of custom handler functions for R objects; see
Details.}
}
\value{
Returns a YAML string which can be loaded using
\code{\link[=yaml.load]{yaml.load()}} or copied into a file for external use.
}
\description{
If you set the \code{omap} option to TRUE, as.yaml will create ordered maps
(or omaps) instead of normal maps.
}
\details{
The \code{column.major} option determines how a data frame is converted. If
TRUE, the data frame is converted into a map of sequences where the name of
each column is a key. If FALSE, the data frame is converted into a sequence
of maps, where each element in the sequence is a row.  You'll probably
almost always want to leave this as TRUE (which is the default), because
using \code{\link[=yaml.load]{yaml.load()}} on the resulting string returns an object
which is much more easily converted into a data frame via
\code{\link[=as.data.frame]{as.data.frame()}}.

You can specify custom handler functions via the \code{handlers} argument.
This argument must be a named list of functions, where the names are R
object class names (i.e., 'numeric', 'data.frame', 'list', etc).  The
function(s) you provide will be passed one argument (the R object) and can
return any R object.  The returned object will be emitted normally.

Character vectors that have a class of \sQuote{verbatim} will not be quoted
in the output YAML document except when the YAML specification requires it.
This means that you cannot do anything that would result in an invalid YAML
document, but you can emit strings that would otherwise be quoted.  This is
useful for changing how logical vectors are emitted (see below for example).

Character vectors that have an attribute of \sQuote{quoted} will be wrapped
in double quotes (see below for example).

You can specify YAML tags for R objects by setting the \sQuote{tag}
attribute to a character vector of length 1.  If you set a tag for a vector,
the tag will be applied to the YAML sequence as a whole, unless the vector
has only 1 element.  If you wish to tag individual elements, you must use a
list of 1-length vectors, each with a tag attribute.  Likewise, if you set a
tag for an object that would be emitted as a YAML mapping (like a data frame
or a named list), it will be applied to the mapping as a whole.  Tags can be
used in conjunction with YAML deserialization functions like
\code{\link[=yaml.load]{yaml.load()}} via custom handlers, however, if you set an internal
tag on an incompatible data type (like \dQuote{!seq 1.0}), errors will occur
when you try to deserialize the document.
}
\examples{

  as.yaml(1:10)
  as.yaml(list(foo=1:10, bar=c("test1", "test2")))
  as.yaml(list(foo=1:10, bar=c("test1", "test2")), indent=3)
  as.yaml(list(foo=1:10, bar=c("test1", "test2")), indent.mapping.sequence=TRUE)
  as.yaml(data.frame(a=1:10, b=letters[1:10], c=11:20))
  as.yaml(list(a=1:2, b=3:4), omap=TRUE)
  as.yaml("multi\nline\nstring")
  as.yaml(function(x) x + 1)
  as.yaml(list(foo=list(list(x = 1, y = 2), list(x = 3, y = 4))))

  # custom handler
  as.yaml(Sys.time(), handlers = list(
    POSIXct = function(x) format(x, "\%Y-\%m-\%d")
  ))

  # custom handler with verbatim output to change how logical vectors are
  # emitted
  as.yaml(c(TRUE, FALSE), handlers = list(
    logical = verbatim_logical))

  # force quotes around a string
  port_def <- "80:80"
  attr(port_def, "quoted") <- TRUE
  x <- list(ports = list(port_def))
  as.yaml(x)

  # custom tag for scalar
  x <- "thing"
  attr(x, "tag") <- "!thing"
  as.yaml(x)

  # custom tag for sequence
  x <- 1:10
  attr(x, "tag") <- "!thing"
  as.yaml(x)

  # custom tag for mapping
  x <- data.frame(a = letters[1:5], b = letters[6:10])
  attr(x, "tag") <- "!thing"
  as.yaml(x)

  # custom tag for each element in a list
  x <- list(1, 2, 3)
  attr(x[[1]], "tag") <- "!a"
  attr(x[[2]], "tag") <- "!b"
  attr(x[[3]], "tag") <- "!c"
  as.yaml(x)

}
\references{
YAML: http://yaml.org

YAML omap type: http://yaml.org/type/omap.html
}
\seealso{
\code{\link[=yaml.load]{yaml.load()}}
}
\author{
Jeremy Stephens \href{mailto:jeremy.f.stephens@vumc.org}{jeremy.f.stephens@vumc.org}
}
\keyword{data}
\keyword{manip}

=pod

=for comment
DO NOT EDIT. This Pod was generated by Swim v0.1.48.
See http://github.com/ingydotnet/swim-pm#readme

=encoding utf8

=head1 NAME

YAML::Error - Error formatting class for YAML modules

=head1 SYNOPSIS

    $self->die('YAML_PARSE_ERR_NO_ANCHOR', $alias);
    $self->warn('YAML_LOAD_WARN_DUPLICATE_KEY');

=head1 DESCRIPTION

This module provides a C<die> and a C<warn> facility.

=head1 AUTHOR

Ingy döt Net <ingy@cpan.org>

=head1 COPYRIGHT

Copyright 2001-2014. Ingy döt Net

This program is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yaml-package.R
\docType{package}
\name{yaml-package}
\alias{yaml}
\alias{yaml-package}
\title{yaml: Methods to Convert R Data to YAML and Back}
\description{
Implements the 'libyaml' 'YAML' 1.1 parser and emitter (\url{https://pyyaml.org/wiki/LibYAML}) for R.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://yaml.r-lib.org}
  \item \url{https://github.com/r-lib/yaml/}
  \item Report bugs at \url{https://github.com/r-lib/yaml/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@posit.co} (\href{https://orcid.org/0000-0003-4757-117X}{ORCID})

Authors:
\itemize{
  \item Jeremy Stephens [contributor]
  \item Kirill Simonov
}

Other contributors:
\itemize{
  \item Shawn Garbett \email{shawn.garbett@vumc.org} (\href{https://orcid.org/0000-0003-4079-5621}{ORCID}) [contributor]
  \item Yihui Xie (\href{https://orcid.org/0000-0003-0645-5666}{ORCID}) [contributor]
  \item Zhuoer Dong [contributor]
  \item Jeffrey Horner [contributor]
  \item reikoch [contributor]
  \item Will Beasley (\href{https://orcid.org/0000-0002-5613-5006}{ORCID}) [contributor]
  \item Brendan O'Connor [contributor]
  \item Michael Quinn [contributor]
  \item Charlie Gao [contributor]
  \item Gregory R. Warnes [contributor]
  \item Zhian N. Kamvar [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_yaml.R
\name{write_yaml}
\alias{write_yaml}
\title{Write a YAML file}
\usage{
write_yaml(x, file, fileEncoding = "UTF-8", ...)
}
\arguments{
\item{x}{The object to be converted.}

\item{file}{Either a character string naming a file or a \link{connection}
open for writing.}

\item{fileEncoding}{Character string: if non-empty declares the encoding to
be used on a file (not a connection) so the character data can be
re-encoded as they are written. See \code{\link[=file]{file()}}.}

\item{\dots}{Arguments to \code{\link[=as.yaml]{as.yaml()}}.}
}
\description{
Write the YAML representation of an R object to a file
}
\details{
If \code{file} is a non-open connection, an attempt is made to open it and
then close it after use.

This function is a convenient wrapper around \code{\link[=as.yaml]{as.yaml()}}.
}
\examples{

\dontrun{
  # writing to a file connection
  filename <- tempfile()
  con <- file(filename, "w")
  write_yaml(data.frame(a=1:10, b=letters[1:10], c=11:20), con)
  close(con)

  # using a filename to specify output file
  write_yaml(data.frame(a=1:10, b=letters[1:10], c=11:20), filename)
}

}
\seealso{
\code{\link[=as.yaml]{as.yaml()}}, \code{\link[=read_yaml]{read_yaml()}},
\code{\link[=yaml.load_file]{yaml.load_file()}}
}
\author{
Jeremy Stephens \href{mailto:jeremy.f.stephens@vumc.org}{jeremy.f.stephens@vumc.org}
}
\keyword{data}
\keyword{manip}

=pod

=for comment
DO NOT EDIT. This Pod was generated by Swim v0.1.43.
See http://github.com/ingydotnet/swim-pm#readme

=encoding utf8

=head1 NAME

YAML::Loader::Base - Base class for YAML Loader classes

=head1 SYNOPSIS

    package YAML::Loader::Something;
    use YAML::Loader::Base -base;

=head1 DESCRIPTION

YAML::Loader::Base is a base class for creating YAML loader classes.

=head1 AUTHOR

Ingy döt Net <ingy@cpan.org>

=head1 COPYRIGHT

Copyright 2001-2014. Ingy döt Net

This program is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.

See L<http://www.perl.com/perl/misc/Artistic.html>

=cut
